@@ -1,6 +1,6 @@
 ;;; installed-scm-file
 
-;;;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002 Free Software Foundation, Inc.
+;;;; Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 ;;;;
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
 ;;;;
 ;;;; You should have received a copy of the GNU General Public License
 ;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;;;; Boston, MA 02111-1307 USA
+;;;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;;;; Boston, MA 02110-1301 USA
 ;;;;
 ;;;; As a special exception, the Free Software Foundation gives permission
 ;;;; for additional uses of the text contained in its release of GUILE.
@@ -258,7 +258,7 @@
 
 (define (environment-module env)
   (let ((closure (and (pair? env) (car (last-pair env)))))
-    (and closure (procedure-property closure 'module))))
+    (and closure (eval-closure-module closure))))
 
 
 ;;; {Records}
@@ -459,7 +459,7 @@
 (define file-exists?
   (if (provided? 'posix)
       (lambda (str)
-	(access? str F_OK))
+	(->bool (false-if-exception (stat str))))
       (lambda (str)
 	(let ((port (catch 'system-error (lambda () (open-file str OPEN_READ))
 			   (lambda args #f))))
@@ -774,6 +774,15 @@
 (define (log10 arg)
   (/ (log arg) (log 10)))
 
+(define (numerator q)
+  (do ((num q (* 2 num)))
+      ((integer? num) num)))
+
+(define (denominator q)
+  (do ((num q (* 2 num))
+       (den (- q q -1) (* 2 den)))
+      ((integer? num) den)))
+
 
 
 ;;; {Reader Extensions}
@@ -868,7 +877,7 @@
 		(left-part (string-append
 			    (with-output-to-string
 			      (lambda ()
-				(map (lambda (x) (display (keyword-symbol x)) (display " "))
+				(map (lambda (x) (display (keyword->symbol x)) (display " "))
 				     opts-proper)))
 			    arg-name))
 		(middle-part (if (and (< (string-length left-part) 30)
@@ -1758,10 +1767,10 @@
 		 (unrecognized kws))
 	     (let* ((interface-args (cadr kws))
 		    (interface (apply resolve-interface interface-args)))
-	       (and (eq? (car kws) 'use-syntax)
-		    (or (symbol? (car spec))
+	       (and (eq? (car kws) #:use-syntax)
+		    (or (symbol? (car interface-args))
 			(error "invalid module name for use-syntax"
-			       spec))
+			       interface-args))
 		    (set-module-transformer!
 		     module
 		     (module-ref interface (car
@@ -2761,8 +2770,8 @@
 	     (set-module-transformer! (current-module)
 				      ,(car (last-pair spec))))
 	   `((set-module-transformer! (current-module) ,spec)))
-     (begin-deprecated
-      (fluid-set! scm:eval-transformer (module-transformer (current-module)))))
+     (fluid-set! scm:eval-transformer (module-transformer (current-module)))
+     *unspecified*)
     (else
      (error "use-syntax can only be used at the top level"))))
 
@@ -2887,13 +2896,17 @@
 ;;;
 ;;; Currently, the following feature identifiers are supported:
 ;;;
-;;;   guile r5rs srfi-0
+;;;   guile r5rs srfi-0 srfi-6
 ;;;
 ;;; Remember to update the features list when adding more SRFIs.
 
 (define %cond-expand-features
   ;; Adjust the above comment when changing this.
-  '(guile r5rs srfi-0))
+  '(guile
+    r5rs
+    srfi-0   ;; cond-expand itself
+    srfi-6   ;; open-input-string etc, in the guile core
+    ))
 
 ;; This table maps module public interfaces to the list of features.
 ;;
