@@ -1,6 +1,6 @@
 ;;;; optargs.scm -- support for optional arguments
 ;;;;
-;;;; 	Copyright (C) 1997, 1998, 1999, 2001, 2002 Free Software Foundation, Inc.
+;;;; 	Copyright (C) 1997, 1998, 1999, 2001, 2002, 2004 Free Software Foundation, Inc.
 ;;;;
 ;;;; This program is free software; you can redistribute it and/or modify
 ;;;; it under the terms of the GNU General Public License as published by
@@ -14,8 +14,8 @@
 ;;;;
 ;;;; You should have received a copy of the GNU General Public License
 ;;;; along with this software; see the file COPYING.  If not, write to
-;;;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;;;; Boston, MA 02111-1307 USA
+;;;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;;;; Boston, MA 02110-1301 USA
 ;;;;
 ;;;; As a special exception, the Free Software Foundation gives permission
 ;;;; for additional uses of the text contained in its release of GUILE.
@@ -150,7 +150,7 @@
 
 (define (let-optional-template REST-ARG BINDINGS BODY let-type)
     (if (null? BINDINGS)
-	`(begin ,@BODY)
+	`(let () ,@BODY)
 	(let-o-k-template REST-ARG BINDINGS BODY let-type
 			  (lambda (optional)
 			    `(,(car optional)
@@ -165,7 +165,7 @@
 
 (define (let-keywords-template REST-ARG ALLOW-OTHER-KEYS? BINDINGS BODY let-type)
     (if (null? BINDINGS)
-	`(begin ,@BODY)
+	`(let () ,@BODY)
 	(let* ((kb-list-gensym (gensym "kb:G"))
 	       (bindfilter (lambda (key)
 			     `(,(car key)
