@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996, 1997, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1995,1996, 1997, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,7 +42,6 @@
 
 
 
-#include <stdio.h>
 #include "libguile/_scm.h"
 #include "libguile/ports.h"
 #include "libguile/smob.h"
@@ -58,7 +57,7 @@
  * SCM_DEFER_INTS). 
  */
 
-static long scm_tc16_arbiter;
+static scm_t_bits scm_tc16_arbiter;
 
 
 #define SCM_ARB_LOCKED(arb)  ((SCM_CELL_WORD_0 (arb)) & (1L << 16))
@@ -66,20 +65,21 @@ static long scm_tc16_arbiter;
 #define SCM_UNLOCK_ARB(arb)  (SCM_SET_CELL_WORD_0 ((arb), scm_tc16_arbiter));
 
 static int 
-prinarb (SCM exp, SCM port, scm_print_state *pstate)
+arbiter_print (SCM exp, SCM port, scm_print_state *pstate)
 {
   scm_puts ("#<arbiter ", port);
   if (SCM_ARB_LOCKED (exp))
     scm_puts ("locked ", port);
-  scm_iprin1 (SCM_CDR (exp), port, pstate);
+  scm_iprin1 (SCM_PACK (SCM_SMOB_DATA (exp)), port, pstate);
   scm_putc ('>', port);
   return !0;
 }
 
 SCM_DEFINE (scm_make_arbiter, "make-arbiter", 1, 0, 0, 
-           (SCM name),
-"Returns an object of type arbiter and name name. Its state is initially unlocked.\n"
-"Arbiters are a way to achieve process synchronization.")
+	    (SCM name),
+	    "Return an object of type arbiter and name @var{name}. Its\n"
+	    "state is initially unlocked.  Arbiters are a way to achieve\n"
+	    "process synchronization.")
 #define FUNC_NAME s_scm_make_arbiter
 {
   SCM_RETURN_NEWSMOB (scm_tc16_arbiter, SCM_UNPACK (name));
@@ -87,11 +87,12 @@ SCM_DEFINE (scm_make_arbiter, "make-arbiter", 1, 0, 0,
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_try_arbiter, "try-arbiter", 1, 0, 0, 
-           (SCM arb),
-"Returns #t and locks arbiter if arbiter was unlocked. Otherwise, returns #f.")
+	    (SCM arb),
+	    "Return @code{#t} and lock the arbiter @var{arb} if the arbiter\n"
+	    "was unlocked. Otherwise, return @code{#f}.")
 #define FUNC_NAME s_scm_try_arbiter
 {
-  SCM_VALIDATE_SMOB (1,arb,arbiter);
+  SCM_VALIDATE_SMOB (1, arb, arbiter);
   SCM_DEFER_INTS;
   if (SCM_ARB_LOCKED(arb))
     arb = SCM_BOOL_F;
@@ -107,12 +108,13 @@ SCM_DEFINE (scm_try_arbiter, "try-arbiter", 1, 0, 0,
 
 
 SCM_DEFINE (scm_release_arbiter, "release-arbiter", 1, 0, 0, 
-           (SCM arb),
-"Returns #t and unlocks arbiter if arbiter was locked. Otherwise, returns #f.")
+	    (SCM arb),
+	    "Return @code{#t} and unlock the arbiter @var{arb} if the\n"
+	    "arbiter was locked. Otherwise, return @code{#f}.")
 #define FUNC_NAME s_scm_release_arbiter
 {
-  SCM_VALIDATE_SMOB (1,arb,arbiter);
-  if (! SCM_ARB_LOCKED(arb))
+  SCM_VALIDATE_SMOB (1, arb, arbiter);
+  if (!SCM_ARB_LOCKED(arb))
     return SCM_BOOL_F;
   SCM_UNLOCK_ARB (arb);
   return SCM_BOOL_T;
@@ -124,8 +126,9 @@ SCM_DEFINE (scm_release_arbiter, "release-arbiter", 1, 0, 0,
 void
 scm_init_arbiters ()
 {
-  scm_tc16_arbiter = scm_make_smob_type_mfpe ("arbiter", 0,
-                                              scm_markcdr, NULL, prinarb, NULL);
+  scm_tc16_arbiter = scm_make_smob_type ("arbiter", 0);
+  scm_set_smob_mark (scm_tc16_arbiter, scm_markcdr);
+  scm_set_smob_print (scm_tc16_arbiter, arbiter_print);
 #include "libguile/arbiters.x"
 }
 
