@@ -1,5 +1,5 @@
 /* Printing of backtraces and error messages
- * Copyright (C) 1996, 1997, 1998, 1999, 2000, 2002 Free Software Foundation
+ * Copyright (C) 1996,1997,1998,1999,2000,2001 Free Software Foundation
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -53,6 +53,9 @@
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
+#ifdef HAVE_IO_H
+#include <io.h>
+#endif
 
 #include "libguile/stacks.h"
 #include "libguile/srcprop.h"
@@ -65,6 +68,7 @@
 
 #include "libguile/validate.h"
 #include "libguile/backtrace.h"
+#include "libguile/filesys.h"
 
 /* {Error reporting and backtraces}
  * (A first approximation.)
@@ -79,25 +83,33 @@
           return SCM_BOOL_F;
 #endif
 
-SCM scm_the_last_stack_fluid;
+SCM scm_the_last_stack_fluid_var;
 
 static void
 display_header (SCM source, SCM port)
 {
-  SCM fname = (SCM_MEMOIZEDP (source)
-	       ? scm_source_property (source, scm_sym_filename)
-	       : SCM_BOOL_F);
-  if (SCM_STRINGP (fname))
+  if (SCM_MEMOIZEDP (source))
     {
-      scm_prin1 (fname, port, 0);
-      scm_putc (':', port);
-      scm_intprint (SCM_INUM (scm_source_property (source, scm_sym_line)) + 1,
-		    10,
-		    port);
-      scm_putc (':', port);
-      scm_intprint (SCM_INUM (scm_source_property (source, scm_sym_column)) + 1,
-		    10,
-		    port);
+      SCM fname = scm_source_property (source, scm_sym_filename);
+      SCM line = scm_source_property (source, scm_sym_line);
+      SCM col = scm_source_property (source, scm_sym_column);
+
+      /* Dirk:FIXME:: Maybe we should store the _port_ rather than the
+       * filename with the source properties?  Then we could in case of
+       * non-file ports give at least some more details than just
+       * "<unnamed port>". */
+      if (SCM_STRINGP (fname))
+	scm_prin1 (fname, port, 0);
+      else
+	scm_puts ("<unnamed port>", port);
+
+      if (!SCM_FALSEP (line) && !SCM_FALSEP (col))
+	{
+	  scm_putc (':', port);
+	  scm_intprint (SCM_INUM (line) + 1, 10, port);
+	  scm_putc (':', port);
+	  scm_intprint (SCM_INUM (col) + 1, 10, port);
+	}
     }
   else
     scm_puts ("ERROR", port);
@@ -108,15 +120,15 @@ display_header (SCM source, SCM port)
 void
 scm_display_error_message (SCM message, SCM args, SCM port)
 {
-  if (SCM_ROSTRINGP (message) && SCM_NFALSEP (scm_list_p (args)))
+  if (SCM_STRINGP (message) && !SCM_FALSEP (scm_list_p (args)))
     {
       scm_simple_format (port, message, args);
       scm_newline (port);
     }
   else
     {
-      scm_prin1 (message, port, 0);
-      scm_putc ('\n', port);
+      scm_display (message, port);
+      scm_newline (port);
     }
 }
 
@@ -129,7 +141,7 @@ display_expression (SCM frame,SCM pname,SCM source,SCM port)
   pstate->fancyp = 1;
   pstate->level = 2;
   pstate->length = 3;
-  if (SCM_ROSTRINGP (pname))
+  if (SCM_SYMBOLP (pname) || SCM_STRINGP (pname))
     {
       if (SCM_FRAMEP (frame)
 	  && SCM_FRAME_EVAL_ARGS_P (frame))
@@ -144,7 +156,7 @@ display_expression (SCM frame,SCM pname,SCM source,SCM port)
 	  scm_iprin1 (scm_unmemoize (source), port, pstate);
 	}
     }
-  else if (SCM_NIMP (source))
+  else if (SCM_MEMOIZEDP (source))
     {
       scm_puts ("In expression ", port);
       pstate->writingp = 1;
@@ -168,8 +180,8 @@ display_error_body (struct display_error_args *a)
 {
   SCM current_frame = SCM_BOOL_F;
   SCM source = SCM_BOOL_F;
-  SCM pname = SCM_BOOL_F;
   SCM prev_frame = SCM_BOOL_F;
+  SCM pname = a->subr;
 
   if (SCM_DEBUGGINGP
       && SCM_STACKP (a->stack)
@@ -180,13 +192,11 @@ display_error_body (struct display_error_args *a)
       prev_frame = SCM_FRAME_PREV (current_frame);
       if (!SCM_MEMOIZEDP (source) && !SCM_FALSEP (prev_frame))
 	source = SCM_FRAME_SOURCE (prev_frame);
-      if (SCM_FRAME_PROC_P (current_frame)
+      if (!SCM_SYMBOLP (pname) && !SCM_STRINGP (pname) && SCM_FRAME_PROC_P (current_frame)
 	  && SCM_EQ_P (scm_procedure_p (SCM_FRAME_PROC (current_frame)), SCM_BOOL_T))
 	pname = scm_procedure_name (SCM_FRAME_PROC (current_frame));
     }
-  if (!SCM_ROSTRINGP (pname))
-    pname = a->subr;
-  if (SCM_ROSTRINGP (pname) || SCM_MEMOIZEDP (source))
+  if (SCM_SYMBOLP (pname) || SCM_STRINGP (pname) || SCM_MEMOIZEDP (source))
     {
       display_header (source, a->port);
       display_expression (current_frame, pname, source, a->port);
@@ -208,7 +218,7 @@ struct display_error_handler_data {
    try to print all objects, we would enter an infinite loop. */
 static SCM
 display_error_handler (struct display_error_handler_data *data,
-		       SCM tag, SCM args)
+		       SCM tag, SCM args SCM_UNUSED)
 {
   SCM print_state = scm_make_print_state ();
   scm_puts ("\nException during displaying of ", data->port);
@@ -219,10 +229,14 @@ display_error_handler (struct display_error_handler_data *data,
   return SCM_UNSPECIFIED;
 }
 
-SCM_DEFINE (scm_display_error, "display-error", 6, 0, 0,
-           (SCM stack, SCM port, SCM subr, SCM message, SCM args, SCM rest),
-"")
-#define FUNC_NAME s_scm_display_error
+
+/* The function scm_i_display_error prints out a detailed error message.  This
+ * function will be called directly within libguile to signal error messages.
+ * No parameter checks will be performed by scm_i_display_error.  Thus, User
+ * code should rather use the function scm_display_error.
+ */
+void
+scm_i_display_error (SCM stack, SCM port, SCM subr, SCM message, SCM args, SCM rest)
 {
   struct display_error_args a;
   struct display_error_handler_data data;
@@ -235,12 +249,31 @@ SCM_DEFINE (scm_display_error, "display-error", 6, 0, 0,
   data.mode = "error";
   data.port = port;
   scm_internal_catch (SCM_BOOL_T,
-		      (scm_catch_body_t) display_error_body, &a,
-		      (scm_catch_handler_t) display_error_handler, &data);
+		      (scm_t_catch_body) display_error_body, &a,
+		      (scm_t_catch_handler) display_error_handler, &data);
+}
+
+
+SCM_DEFINE (scm_display_error, "display-error", 6, 0, 0,
+	    (SCM stack, SCM port, SCM subr, SCM message, SCM args, SCM rest),
+	    "Display an error message to the output port @var{port}.\n"
+	    "@var{stack} is the saved stack for the error, @var{subr} is\n"
+	    "the name of the procedure in which the error occurred and\n"
+	    "@var{message} is the actual error message, which may contain\n"
+	    "formatting instructions. These will format the arguments in\n"
+	    "the list @var{args} accordingly.  @var{rest} is currently\n"
+	    "ignored.")
+#define FUNC_NAME s_scm_display_error
+{
+  SCM_VALIDATE_OUTPUT_PORT (2, port);
+
+  scm_i_display_error (stack, port, subr, message, args, rest);
+
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
+
 typedef struct {
   int level;
   int length;
@@ -258,7 +291,9 @@ static print_params_t *print_params = default_print_params;
 #ifdef GUILE_DEBUG
 SCM_DEFINE (scm_set_print_params_x, "set-print-params!", 1, 0, 0,
            (SCM params),
-"")
+	    "Set the print parameters to the values from @var{params}.\n"
+	    "@var{params} must be a list of two-element lists which must\n"
+	    "hold two integer values.")
 #define FUNC_NAME s_scm_set_print_params_x
 {
   int i;
@@ -267,7 +302,7 @@ SCM_DEFINE (scm_set_print_params_x, "set-print-params!", 1, 0, 0,
   print_params_t *new_params;
 
   SCM_VALIDATE_NONEMPTYLIST_COPYLEN (2, params, n);
-  for (ls = params; SCM_NNULLP (ls); ls = SCM_CDR (ls))
+  for (ls = params; !SCM_NULLP (ls); ls = SCM_CDR (ls))
     SCM_ASSERT (scm_ilength (SCM_CAR (params)) == 2
 		&& SCM_INUMP (SCM_CAAR (ls))
 		&& SCM_INUM (SCM_CAAR (ls)) >= 0
@@ -306,7 +341,7 @@ display_frame_expr (char *hdr,SCM exp,char *tlr,int indentation,SCM sport,SCM po
 {
   SCM string;
   int i = 0, n;
-  scm_ptob_descriptor *ptob = scm_ptobs + SCM_PTOBNUM (sport);
+  scm_t_ptob_descriptor *ptob = scm_ptobs + SCM_PTOBNUM (sport);
   do
     {
       pstate->length = print_params[i].length;
@@ -331,27 +366,27 @@ display_frame_expr (char *hdr,SCM exp,char *tlr,int indentation,SCM sport,SCM po
   string = scm_strport_to_string (sport);
   /* Remove control characters */
   for (i = 0; i < n; ++i)
-    if (iscntrl (SCM_CHARS (string)[i]))
-      SCM_CHARS (string)[i] = ' ';
+    if (iscntrl (SCM_STRING_CHARS (string)[i]))
+      SCM_STRING_CHARS (string)[i] = ' ';
   /* Truncate */
   if (indentation + n > SCM_BACKTRACE_WIDTH)
     {
       n = SCM_BACKTRACE_WIDTH - indentation;
-      SCM_CHARS (string)[n - 1] = '$';
+      SCM_STRING_CHARS (string)[n - 1] = '$';
     }
       
-  scm_lfwrite (SCM_CHARS (string), n, port);
+  scm_lfwrite (SCM_STRING_CHARS (string), n, port);
 }
 
 static void
 display_application (SCM frame,int indentation,SCM sport,SCM port,scm_print_state *pstate)
 {
   SCM proc = SCM_FRAME_PROC (frame);
-  SCM name = (SCM_NFALSEP (scm_procedure_p (proc))
+  SCM name = (!SCM_FALSEP (scm_procedure_p (proc))
 	      ? scm_procedure_name (proc)
 	      : SCM_BOOL_F);
   display_frame_expr ("[",
-		      scm_cons (SCM_NFALSEP (name) ? name : proc,
+		      scm_cons (!SCM_FALSEP (name) ? name : proc,
 				SCM_FRAME_ARGS (frame)),
 		      SCM_FRAME_EVAL_ARGS_P (frame) ? " ..." : "]",
 		      indentation,
@@ -362,7 +397,9 @@ display_application (SCM frame,int indentation,SCM sport,SCM port,scm_print_stat
 
 SCM_DEFINE (scm_display_application, "display-application", 1, 2, 0, 
            (SCM frame, SCM port, SCM indent),
-"")
+	    "Display a procedure application @var{frame} to the output port\n"
+	    "@var{port}. @var{indent} specifies the indentation of the\n"
+	    "output.")
 #define FUNC_NAME s_scm_display_application
 {
   SCM_VALIDATE_FRAME (1,frame);
@@ -402,6 +439,92 @@ SCM_DEFINE (scm_display_application, "display-application", 1, 2, 0,
 }
 #undef FUNC_NAME
 
+SCM_SYMBOL (sym_base, "base");
+
+static void
+display_backtrace_get_file_line (SCM frame, SCM *file, SCM *line)
+{
+  SCM source = SCM_FRAME_SOURCE (frame);
+  *file = SCM_MEMOIZEDP (source) ? scm_source_property (source, scm_sym_filename) : SCM_BOOL_F;
+  *line = (SCM_MEMOIZEDP (source)) ? scm_source_property (source, scm_sym_line) : SCM_BOOL_F;
+}
+
+static void
+display_backtrace_file (frame, last_file, port, pstate)
+     SCM frame;
+     SCM *last_file;
+     SCM port;
+     scm_print_state *pstate;
+{
+  SCM file, line;
+
+  display_backtrace_get_file_line (frame, &file, &line);
+
+  if (file == *last_file)
+    return;
+
+  *last_file = file;
+
+  scm_puts ("In ", port);
+  if (SCM_FALSEP (file))
+    if (SCM_FALSEP (line))
+      scm_puts ("unknown file", port);
+    else
+      scm_puts ("current input", port);
+  else
+    {
+      pstate->writingp = 0;
+      scm_iprin1 (file, port, pstate);
+      pstate->writingp = 1;
+    }
+  scm_puts (":\n", port);
+}
+
+static void
+display_backtrace_file_and_line (SCM frame, SCM port, scm_print_state *pstate)
+{
+  SCM file, line;
+
+  display_backtrace_get_file_line (frame, &file, &line);
+
+  if (SCM_EQ_P (SCM_SHOW_FILE_NAME, sym_base))
+    {
+      if (file == SCM_BOOL_F)
+	{
+	  if (line == SCM_BOOL_F)
+	    scm_putc ('?', port);
+	  else
+	    scm_puts ("<stdin>", port);
+	}
+      else
+	{
+	  pstate -> writingp = 0;
+#ifdef HAVE_POSIX
+	  scm_iprin1 (SCM_STRINGP (file) ? scm_basename (file, SCM_UNDEFINED) : file,
+		      port, pstate);
+#else
+	  scm_iprin1 (file, port, pstate);
+#endif
+	  pstate -> writingp = 1;
+	}
+
+      scm_putc (':', port);
+    }
+  else if (line != SCM_BOOL_F)
+    {
+      int i, j=0;
+      for (i = SCM_INUM (line)+1; i > 0; i = i/10, j++)
+	;
+      indent (4-j, port);
+    }
+
+  if (line == SCM_BOOL_F)
+    scm_puts ("   ?", port);
+  else
+    scm_intprint (SCM_INUM (line) + 1, 10, port);
+  scm_puts (": ", port);
+}
+
 static void
 display_frame (SCM frame,int nfield,int indentation,SCM sport,SCM port,scm_print_state *pstate)
 {
@@ -414,6 +537,10 @@ display_frame (SCM frame,int nfield,int indentation,SCM sport,SCM port,scm_print
       scm_puts ("...\n", port);
     }
 
+  /* display file name and line number */
+  if (!SCM_FALSEP (SCM_SHOW_FILE_NAME))
+    display_backtrace_file_and_line (frame, port, pstate);
+
   /* Check size of frame number. */
   n = SCM_FRAME_NUMBER (frame);
   for (i = 0, j = n; j > 0; ++i) j /= 10;
@@ -469,25 +596,20 @@ struct display_backtrace_args {
 };
 
 static SCM
-display_backtrace_body(struct display_backtrace_args *a)
+display_backtrace_body (struct display_backtrace_args *a)
 #define FUNC_NAME "display_backtrace_body"
 {
   int n_frames, beg, end, n, i, j;
   int nfield, indent_p, indentation;
   SCM frame, sport, print_state;
+  SCM last_file;
   scm_print_state *pstate;
 
   a->port = SCM_COERCE_OUTPORT (a->port);
 
   /* Argument checking and extraction. */
-  SCM_ASSERT (SCM_STACKP (a->stack),
-	      a->stack,
-	      SCM_ARG1,
-	      s_display_backtrace);
-  SCM_ASSERT (SCM_OPOUTPORTP (a->port),
-	      a->port,
-	      SCM_ARG2,
-	      s_display_backtrace);
+  SCM_VALIDATE_STACK (1, a->stack);
+  SCM_VALIDATE_OPOUTPORT (2, a->port);
   n_frames = SCM_INUM (scm_stack_length (a->stack));
   n = SCM_INUMP (a->depth) ? SCM_INUM (a->depth) : SCM_BACKTRACE_DEPTH;
   if (SCM_BACKWARDS_P)
@@ -536,6 +658,8 @@ display_backtrace_body(struct display_backtrace_args *a)
     indent_p = 0;
   else
     {
+      unsigned int j;
+
       indent_p = 1;
       frame = scm_stack_ref (a->stack, SCM_MAKINUM (beg));
       for (i = 0, j = 0; i < n; ++i)
@@ -561,22 +685,32 @@ display_backtrace_body(struct display_backtrace_args *a)
   /* Print frames. */
   frame = scm_stack_ref (a->stack, SCM_MAKINUM (beg));
   indentation = 1;
-  display_frame (frame, nfield, indentation, sport, a->port, pstate);
-  for (i = 1; i < n; ++i)
+  last_file = SCM_UNDEFINED;
+  for (i = 0; i < n; ++i)
     {
+      if (!SCM_EQ_P (SCM_SHOW_FILE_NAME, sym_base))
+	display_backtrace_file (frame, &last_file, a->port, pstate);
+
+      display_frame (frame, nfield, indentation, sport, a->port, pstate);
       if (indent_p && SCM_FRAME_EVAL_ARGS_P (frame))
 	++indentation;
-      frame = SCM_BACKWARDS_P ? SCM_FRAME_PREV (frame) : SCM_FRAME_NEXT (frame);
-      display_frame (frame, nfield, indentation, sport, a->port, pstate);
+      frame = (SCM_BACKWARDS_P ? 
+	       SCM_FRAME_PREV (frame) : SCM_FRAME_NEXT (frame));
     }
 
+  scm_remember_upto_here_1 (print_state);
+  
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
 SCM_DEFINE (scm_display_backtrace, "display-backtrace", 2, 2, 0, 
-           (SCM stack, SCM port, SCM first, SCM depth),
-"")
+	    (SCM stack, SCM port, SCM first, SCM depth),
+	    "Display a backtrace to the output port @var{port}. @var{stack}\n"
+	    "is the stack to take the backtrace from, @var{first} specifies\n"
+	    "where in the stack to start and @var{depth} how much frames\n"
+	    "to display. Both @var{first} and @var{depth} can be @code{#f},\n"
+	    "which means that default values will be used.")
 #define FUNC_NAME s_scm_display_backtrace
 {
   struct display_backtrace_args a;
@@ -588,21 +722,23 @@ SCM_DEFINE (scm_display_backtrace, "display-backtrace", 2, 2, 0,
   data.mode = "backtrace";
   data.port = port;
   scm_internal_catch (SCM_BOOL_T,
-		      (scm_catch_body_t) display_backtrace_body, &a,
-		      (scm_catch_handler_t) display_error_handler, &data);
+		      (scm_t_catch_body) display_backtrace_body, &a,
+		      (scm_t_catch_handler) display_error_handler, &data);
   return SCM_UNSPECIFIED;
 }
 #undef FUNC_NAME
 
-SCM_VCELL (scm_has_shown_backtrace_hint_p_var, "has-shown-backtrace-hint?");
+SCM_VARIABLE (scm_has_shown_backtrace_hint_p_var, "has-shown-backtrace-hint?");
 
 SCM_DEFINE (scm_backtrace, "backtrace", 0, 0, 0, 
-           (),
-"")
+	    (),
+	    "Display a backtrace of the stack saved by the last error\n"
+	    "to the current output port.")
 #define FUNC_NAME s_scm_backtrace
 {
-  SCM the_last_stack = scm_fluid_ref (SCM_CDR (scm_the_last_stack_fluid));
-  if (SCM_NFALSEP (the_last_stack))
+  SCM the_last_stack =
+    scm_fluid_ref (SCM_VARIABLE_REF (scm_the_last_stack_fluid_var));
+  if (!SCM_FALSEP (the_last_stack))
     {
       scm_newline (scm_cur_outp);
       scm_puts ("Backtrace:\n", scm_cur_outp);
@@ -611,14 +747,14 @@ SCM_DEFINE (scm_backtrace, "backtrace", 0, 0, 0,
 			     SCM_UNDEFINED,
 			     SCM_UNDEFINED);
       scm_newline (scm_cur_outp);
-      if (SCM_FALSEP (SCM_CDR (scm_has_shown_backtrace_hint_p_var))
+      if (SCM_FALSEP (SCM_VARIABLE_REF (scm_has_shown_backtrace_hint_p_var))
 	  && !SCM_BACKTRACE_P)
 	{
 	  scm_puts ("Type \"(debug-enable 'backtrace)\" if you would like "
 		    "a backtrace\n"
 		    "automatically if an error occurs in the future.\n",
 		    scm_cur_outp);
-	  SCM_SETCDR (scm_has_shown_backtrace_hint_p_var, SCM_BOOL_T);
+	  SCM_VARIABLE_SET (scm_has_shown_backtrace_hint_p_var, SCM_BOOL_T);
 	}
     }
   else
@@ -635,7 +771,7 @@ void
 scm_init_backtrace ()
 {
   SCM f = scm_make_fluid ();
-  scm_the_last_stack_fluid = scm_sysintern ("the-last-stack", f);
+  scm_the_last_stack_fluid_var = scm_c_define ("the-last-stack", f);
 
 #include "libguile/backtrace.x"
 }
