@@ -1,6 +1,7 @@
 /* dynl.c - dynamic linking
  *
- * Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000, 2001 Free Software Foundation, Inc.
+ * Copyright (C) 1990, 91, 92, 93, 94, 95, 96, 97, 98, 99, 2000, 2001, 2003
+ * Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +15,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -237,13 +238,13 @@ SCM_DEFINE (scm_clear_registered_modules, "c-clear-registered-modules", 0, 0, 0,
 
 #ifdef DYNAMIC_LINKING
 
-#include "libltdl/ltdl.h"
+#include "guile-ltdl.h"
 
 static void *
 sysdep_dynl_link (const char *fname, const char *subr)
 {
-  lt_dlhandle handle;
-  handle = lt_dlopenext (fname);
+  scm_lt_dlhandle handle;
+  handle = scm_lt_dlopenext (fname);
   if (NULL == handle)
     {
       SCM fn;
@@ -251,7 +252,7 @@ sysdep_dynl_link (const char *fname, const char *subr)
 
       SCM_ALLOW_INTS;
       fn = scm_makfrom0str (fname);
-      msg = scm_makfrom0str (lt_dlerror ());
+      msg = scm_makfrom0str (scm_lt_dlerror ());
       scm_misc_error (subr, "file: ~S, message: ~S", scm_list_2 (fn, msg));
     }
   return (void *) handle;
@@ -260,10 +261,10 @@ sysdep_dynl_link (const char *fname, const char *subr)
 static void
 sysdep_dynl_unlink (void *handle, const char *subr)
 {
-  if (lt_dlclose ((lt_dlhandle) handle))
+  if (scm_lt_dlclose ((scm_lt_dlhandle) handle))
     {
       SCM_ALLOW_INTS;
-      scm_misc_error (subr, (char *) lt_dlerror (), SCM_EOL);
+      scm_misc_error (subr, (char *) scm_lt_dlerror (), SCM_EOL);
     }
 }
    
@@ -272,11 +273,11 @@ sysdep_dynl_func (const char *symb, void *handle, const char *subr)
 {
   void *fptr;
 
-  fptr = lt_dlsym ((lt_dlhandle) handle, symb);
+  fptr = scm_lt_dlsym ((scm_lt_dlhandle) handle, symb);
   if (!fptr)
     {
       SCM_ALLOW_INTS;
-      scm_misc_error (subr, (char *) lt_dlerror (), SCM_EOL);
+      scm_misc_error (subr, (char *) scm_lt_dlerror (), SCM_EOL);
     }
   return fptr;
 }
@@ -284,7 +285,7 @@ sysdep_dynl_func (const char *symb, void *handle, const char *subr)
 static void
 sysdep_dynl_init ()
 {
-  lt_dlinit ();
+  scm_lt_dlinit ();
 }
 
 #else
@@ -394,7 +395,7 @@ SCM_DEFINE (scm_dynamic_unlink, "dynamic-unlink", 1, 0, 0,
   /*fixme* GC-problem */
   SCM_VALIDATE_SMOB (SCM_ARG1, dobj, dynamic_obj);
   if (DYNL_HANDLE (dobj) == NULL) {
-    SCM_MISC_ERROR ("Already unlinked: ~S", dobj);
+    SCM_MISC_ERROR ("Already unlinked: ~S", scm_list_1 (dobj));
   } else {
     SCM_DEFER_INTS;
     sysdep_dynl_unlink (DYNL_HANDLE (dobj), FUNC_NAME);
