@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995, 1996, 1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,7 +42,6 @@
 
 
 
-#include <stdio.h>
 #ifdef HAVE_STRING_H
 #include <string.h>
 #endif
@@ -56,22 +55,26 @@
 
 
 
-static SCM features;
+static SCM features_var;
 
 
 void
 scm_add_feature (const char *str)
 {
-  SCM old = SCM_CDR (features);
-  SCM new = scm_cons (SCM_CAR (scm_intern (str, strlen (str))), old);
-  SCM_SETCDR (features, new);
+  SCM old = SCM_VARIABLE_REF (features_var);
+  SCM new = scm_cons (scm_str2symbol (str), old);
+  SCM_VARIABLE_SET (features_var, new);
 }
 
 
 
 SCM_DEFINE (scm_program_arguments, "program-arguments", 0, 0, 0, 
-           (),
-"")
+	    (),
+	    "@deffnx {Scheme Procedure} command-line\n"
+	    "Return the list of command line arguments passed to Guile, as a list of\n"
+	    "strings.  The list includes the invoked program name, which is usually\n"
+	    "@code{\"guile\"}, but excludes switches and parameters for command line\n"
+	    "options like @code{-e} and @code{-l}.")
 #define FUNC_NAME s_scm_program_arguments
 {
   return scm_progargs;
@@ -98,7 +101,7 @@ scm_set_program_arguments (int argc, char **argv, char *first)
 void
 scm_init_feature()
 {
-  features = scm_sysintern ("*features*", SCM_EOL);
+  features_var = scm_c_define ("*features*", SCM_EOL);
 #ifdef SCM_RECKLESS
   scm_add_feature("reckless");
 #endif
@@ -121,7 +124,7 @@ scm_init_feature()
   scm_add_feature ("threads");
 #endif
   
-  scm_sysintern ("char-code-limit", SCM_MAKINUM (SCM_CHAR_CODE_LIMIT));
+  scm_c_define ("char-code-limit", SCM_MAKINUM (SCM_CHAR_CODE_LIMIT));
 
 #include "libguile/feature.x"
 }
