@@ -1,5 +1,5 @@
 /* GDB interface for Guile
- * Copyright (C) 1996, 1997, 1999, 2000, 2002 Free Software Foundation
+ * Copyright (C) 1996,1997,1999,2000,2001 Free Software Foundation, Inc.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -13,8 +13,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -48,18 +48,18 @@
 #include "libguile/_scm.h"
 
 #include <stdio.h>
+#include <string.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
 
-#include "libguile/tag.h"
 #include "libguile/strports.h"
 #include "libguile/read.h"
 #include "libguile/eval.h"
 #include "libguile/chars.h"
 #include "libguile/modules.h"
 #include "libguile/ports.h"
-#include "libguile/root.h"
+#include "libguile/fluids.h"
 #include "libguile/strings.h"
 #include "libguile/init.h"
 
@@ -81,9 +81,9 @@
 
 
 /* Be carefull when this macro is true.
-   scm_gc_heap_lock is set during gc.
+   scm_gc_running_p is set during gc.
  */
-#define SCM_GC_P (scm_gc_heap_lock)
+#define SCM_GC_P (scm_gc_running_p)
 
 /* Macros that encapsulate blocks of code which can be called by the
  * debugger.
@@ -104,8 +104,6 @@ do { \
 } while (0)
 
 
-#define MSG_GUILE_NOT_INITIALIZED "*** Guile not initialized ***"
-
 #define RESET_STRING { gdb_output_length = 0; }
 
 #define SEND_STRING(str) \
@@ -144,14 +142,14 @@ static void
 unmark_port (SCM port)
 {
   SCM stream, string;
-  port_mark_p = SCM_GC8MARKP (port);
-  SCM_CLRGC8MARK (port);
+  port_mark_p = SCM_GCMARKP (port);
+  SCM_CLRGCMARK (port);
   stream = SCM_PACK (SCM_STREAM (port));
   stream_mark_p = SCM_GCMARKP (stream);
   SCM_CLRGCMARK (stream);
   string = SCM_CDR (stream);
-  string_mark_p = SCM_GC8MARKP (string);
-  SCM_CLRGC8MARK (string);
+  string_mark_p = SCM_GCMARKP (string);
+  SCM_CLRGCMARK (string);
 }
 
 
@@ -160,9 +158,9 @@ remark_port (SCM port)
 {
   SCM stream = SCM_PACK (SCM_STREAM (port));
   SCM string = SCM_CDR (stream);
-  if (string_mark_p) SCM_SETGC8MARK (string);
+  if (string_mark_p) SCM_SETGCMARK (string);
   if (stream_mark_p) SCM_SETGCMARK (stream);
-  if (port_mark_p) SCM_SETGC8MARK (port);
+  if (port_mark_p) SCM_SETGCMARK (port);
 }
 
 
@@ -213,8 +211,8 @@ gdb_read (char *str)
   scm_truncate_file (gdb_input_port, SCM_UNDEFINED);
   scm_seek (gdb_input_port, SCM_INUM0, SCM_MAKINUM (SEEK_SET));
   /* Read one object */
-  tok_buf_mark_p = SCM_GC8MARKP (tok_buf);
-  SCM_CLRGC8MARK (tok_buf);
+  tok_buf_mark_p = SCM_GCMARKP (tok_buf);
+  SCM_CLRGCMARK (tok_buf);
   ans = scm_lreadr (&tok_buf, gdb_input_port, &ans);
   if (SCM_GC_P)
     {
@@ -231,7 +229,7 @@ gdb_read (char *str)
     scm_permanent_object (ans);
 exit:
   if (tok_buf_mark_p)
-    SCM_SETGC8MARK (tok_buf);
+    SCM_SETGCMARK (tok_buf);
   remark_port (gdb_input_port);
   SCM_END_FOREIGN_BLOCK;
   return status;
@@ -254,7 +252,7 @@ gdb_eval (SCM exp)
     }
   SCM_BEGIN_FOREIGN_BLOCK;
   {
-    SCM env = scm_top_level_env (SCM_CDR (scm_top_level_lookup_closure_var));
+    SCM env = scm_top_level_env (SCM_TOP_LEVEL_LOOKUP_CLOSURE);
     gdb_result = scm_permanent_object (scm_ceval (exp, env));
   }
   SCM_END_FOREIGN_BLOCK;
@@ -276,7 +274,7 @@ gdb_print (SCM obj)
       scm_write (obj, gdb_output_port);
       scm_truncate_file (gdb_output_port, SCM_UNDEFINED);
       {
-	scm_port *pt = SCM_PTAB_ENTRY (gdb_output_port);
+	scm_t_port *pt = SCM_PTAB_ENTRY (gdb_output_port);
 
 	scm_flush (gdb_output_port);
 	*(pt->write_buf + pt->read_buf_size) = 0;
@@ -299,10 +297,8 @@ gdb_binding (SCM name, SCM value)
     }
   SCM_BEGIN_FOREIGN_BLOCK;
   {
-    SCM vcell = scm_sym2vcell (name,
-			       SCM_CDR (scm_top_level_lookup_closure_var),
-			       SCM_BOOL_T);
-    SCM_SETCDR (vcell, value);
+    SCM var = scm_sym2var (name, SCM_TOP_LEVEL_LOOKUP_CLOSURE, SCM_BOOL_T);
+    SCM_VARIABLE_SET (var, value);
   }
   SCM_END_FOREIGN_BLOCK;
   return 0;
@@ -328,7 +324,7 @@ scm_init_gdbint ()
 			s);
   gdb_input_port = scm_permanent_object (port);
 
-  tok_buf = scm_permanent_object (scm_makstr (30L, 0));
+  tok_buf = scm_permanent_object (scm_allocate_string (30));
 }
 
 /*
