@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -46,7 +46,7 @@
 #include "libguile/__scm.h"
 
 
-extern int scm_tc16_malloc;
+extern scm_t_bits scm_tc16_malloc;
 
 #define SCM_MALLOCP(X)  (SCM_TYP16 (X) == scm_tc16_malloc)
 #define SCM_MALLOCDATA(obj) ((char *) SCM_CELL_WORD_1 (obj))
@@ -54,7 +54,7 @@ extern int scm_tc16_malloc;
 
 
 
-extern SCM scm_malloc_obj (scm_sizet n);
+extern SCM scm_malloc_obj (size_t n);
 extern void scm_init_mallocs (void);
 
 #endif  /* MALLOCSH */
