@@ -2,7 +2,7 @@
 
 #ifndef POSIXH
 #define POSIXH
-/*	Copyright (C) 1995, 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
+/*	Copyright (C) 1995, 1996, 1997, 1998, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -85,6 +85,7 @@ extern SCM scm_fork (void);
 extern SCM scm_uname (void);
 extern SCM scm_environ (SCM env);
 extern SCM scm_tmpnam (void);
+extern SCM scm_mkstemp (SCM tmpl);
 extern SCM scm_open_pipe (SCM pipestr, SCM modes);
 extern SCM scm_close_pipe (SCM port);
 extern SCM scm_utime (SCM pathname, SCM actime, SCM modtime);
@@ -95,6 +96,16 @@ extern SCM scm_setlocale (SCM category, SCM locale);
 extern SCM scm_mknod (SCM path, SCM type, SCM perms, SCM dev);
 extern SCM scm_nice (SCM incr);
 extern SCM scm_sync (void);
+extern SCM scm_crypt (SCM key, SCM salt);
+extern SCM scm_chroot (SCM path);
+extern SCM scm_getlogin (void);
+extern SCM scm_cuserid (void);
+extern SCM scm_getpriority (SCM which, SCM who);
+extern SCM scm_setpriority (SCM which, SCM who, SCM prio);
+extern SCM scm_getpass (SCM prompt);
+extern SCM scm_flock (SCM file, SCM operation);
+extern SCM scm_sethostname (SCM name);
+extern SCM scm_gethostname (void);
 extern void scm_init_posix (void);
 
 #endif  /* POSIXH */
