@@ -1,4 +1,4 @@
-/*	Copyright (C) 1996, 1998, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1998,2000,2001,2002,2004,2005 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -49,8 +49,8 @@
 
 
 
-#include <stdio.h>
 #include "libguile/_scm.h"
+#include "libguile/strings.h"
 #include "libguile/unif.h"
 #include "libguile/smob.h"
 #include "libguile/chars.h"
@@ -127,7 +127,6 @@ do { type *v0 = (type*)SCM_VELTS (ra0);\
      IVDEP (ra0 != ra1, \
 	    for (; n-- > 0; i0 += inc0, i1 += inc1) \
 	       v0[i0] OPERATOR v1[i1];) \
-     break; \
 } while (0)
 
 /* This macro is used for all but binary division and
@@ -141,14 +140,12 @@ do { type (*v0)[2] = (type (*)[2]) SCM_VELTS (ra0);\
 	       v0[i0][0] OPERATOR v1[i1][0]; \
 	       v0[i0][1] OPERATOR v1[i1][1]; \
             }) \
-     break; \
 } while (0)
 
 #define UNARY_ELTS_CODE(OPERATOR, type) \
 	  do { type *v0 = (type *) SCM_VELTS (ra0);\
 	    for (; n-- > 0; i0 += inc0) \
 	      v0[i0] OPERATOR v0[i0];\
-	    break;\
 	  } while (0)
 
 
@@ -164,10 +161,10 @@ do { type (*v0)[2] = (type (*)[2]) SCM_VELTS (ra0);\
 	    break;\
 	  } while (0)
 
-static scm_sizet 
+static unsigned long
 cind (SCM ra, SCM inds)
 {
-  scm_sizet i;
+  unsigned long i;
   int k;
   long *ve = (long*) SCM_VELTS (inds);
   if (!SCM_ARRAYP (ra))
@@ -191,10 +188,10 @@ int
 scm_ra_matchp (SCM ra0, SCM ras)
 {
   SCM ra1;
-  scm_array_dim dims;
-  scm_array_dim *s0 = &dims;
-  scm_array_dim *s1;
-  scm_sizet bas0 = 0;
+  scm_t_array_dim dims;
+  scm_t_array_dim *s0 = &dims;
+  scm_t_array_dim *s1;
+  unsigned long bas0 = 0;
   int i, ndim = 1;
   int exact = 2			/* 4 */ ;	/* Don't care about values >2 (yet?) */
   if (SCM_IMP (ra0)) return 0;
@@ -218,7 +215,7 @@ scm_ra_matchp (SCM ra0, SCM ras)
     case scm_tc7_cvect:
       s0->lbnd = 0;
       s0->inc = 1;
-      s0->ubnd = (long) SCM_LENGTH (ra0) - 1;
+      s0->ubnd = SCM_INUM (scm_uniform_vector_length (ra0)) - 1;
       break;
     case scm_tc7_smob:
       if (!SCM_ARRAYP (ra0))
@@ -252,25 +249,32 @@ scm_ra_matchp (SCM ra0, SCM ras)
 	case scm_tc7_fvect:
 	case scm_tc7_dvect:
 	case scm_tc7_cvect:
-	  if (1 != ndim)
-	    return 0;
-	  switch (exact)
-	    {
-	    case 4:
-	      if (0 != bas0)
-		exact = 3;
-	    case 3:
-	      if (1 != s0->inc)
-		exact = 2;
-	    case 2:
-	      if ((0 == s0->lbnd) && (s0->ubnd == SCM_LENGTH (ra1) - 1))
-		break;
-	      exact = 1;
-	    case 1:
-	      if (s0->lbnd < 0 || s0->ubnd >= SCM_LENGTH (ra1))
-		return 0;
-	    }
-	  break;
+	  {
+	    unsigned long int length;
+
+	    if (1 != ndim)
+	      return 0;
+
+	    length = SCM_INUM (scm_uniform_vector_length (ra1));
+
+	    switch (exact)
+	      {
+	      case 4:
+		if (0 != bas0)
+		  exact = 3;
+	      case 3:
+		if (1 != s0->inc)
+		  exact = 2;
+	      case 2:
+		if ((0 == s0->lbnd) && (s0->ubnd == length - 1))
+		  break;
+		exact = 1;
+	      case 1:
+		if (s0->lbnd < 0 || s0->ubnd >= length)
+		  return 0;
+	      }
+	    break;
+	  }
 	case scm_tc7_smob:
 	  if (!SCM_ARRAYP (ra1) || ndim != SCM_ARRAY_NDIM (ra1))
 	    return 0;
@@ -319,7 +323,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
     {
     default:
     case 0:
-      scm_wta (ra0, "array shape mismatch", what);
+      scm_misc_error (what, "array shape mismatch: ~S", ra0);
     case 2:
     case 3:
     case 4:			/* Try unrolling arrays */
@@ -330,10 +334,11 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
       if (SCM_IMP (vra0)) goto gencase;
       if (!SCM_ARRAYP (vra0))
 	{
+	  unsigned long int length = SCM_INUM (scm_uniform_vector_length (vra0));
 	  vra1 = scm_make_ra (1);
 	  SCM_ARRAY_BASE (vra1) = 0;
 	  SCM_ARRAY_DIMS (vra1)->lbnd = 0;
-	  SCM_ARRAY_DIMS (vra1)->ubnd = SCM_LENGTH (vra0) - 1;
+	  SCM_ARRAY_DIMS (vra1)->ubnd = length - 1;
 	  SCM_ARRAY_DIMS (vra1)->inc = 1;
 	  SCM_ARRAY_V (vra1) = vra0;
 	  vra0 = vra1;
@@ -387,9 +392,10 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
       }
     else
       {
+	unsigned long int length = SCM_INUM (scm_uniform_vector_length (ra0));
 	kmax = 0;
 	SCM_ARRAY_DIMS (vra0)->lbnd = 0;
-	SCM_ARRAY_DIMS (vra0)->ubnd = SCM_LENGTH (ra0) - 1;
+	SCM_ARRAY_DIMS (vra0)->ubnd = length - 1;
 	SCM_ARRAY_DIMS (vra0)->inc = 1;
 	SCM_ARRAY_BASE (vra0) = 0;
 	SCM_ARRAY_V (vra0) = ra0;
@@ -452,7 +458,7 @@ scm_ramapc (int (*cproc)(), SCM data, SCM ra0, SCM lra, const char *what)
 
 SCM_DEFINE (scm_array_fill_x, "array-fill!", 2, 0, 0,
 	    (SCM ra, SCM fill),
-	    "Stores @var{fill} in every element of @var{array}.  The value returned\n"
+	    "Store @var{fill} in every element of @var{array}.  The value returned\n"
 	    "is unspecified.")
 #define FUNC_NAME s_scm_array_fill_x
 {
@@ -464,13 +470,13 @@ SCM_DEFINE (scm_array_fill_x, "array-fill!", 2, 0, 0,
 /* to be used as cproc in scm_ramapc to fill an array dimension with
    "fill". */
 int 
-scm_array_fill_int (SCM ra, SCM fill, SCM ignore)
+scm_array_fill_int (SCM ra, SCM fill, SCM ignore SCM_UNUSED)
 #define FUNC_NAME s_scm_array_fill_x
 {
-  scm_sizet i;
-  scm_sizet n = SCM_ARRAY_DIMS (ra)->ubnd - SCM_ARRAY_DIMS (ra)->lbnd + 1;
+  unsigned long i;
+  unsigned long n = SCM_ARRAY_DIMS (ra)->ubnd - SCM_ARRAY_DIMS (ra)->lbnd + 1;
   long inc = SCM_ARRAY_DIMS (ra)->inc;
-  scm_sizet base = SCM_ARRAY_BASE (ra);
+  unsigned long base = SCM_ARRAY_BASE (ra);
 
   ra = SCM_ARRAY_V (ra);
   switch SCM_TYP7 (ra)
@@ -487,21 +493,21 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore)
     case scm_tc7_string:
       SCM_ASRTGO (SCM_CHARP (fill), badarg2);
       for (i = base; n--; i += inc)
-	SCM_CHARS (ra)[i] = SCM_CHAR (fill);
+	SCM_STRING_CHARS (ra)[i] = SCM_CHAR (fill);
       break;
     case scm_tc7_byvect:
       if (SCM_CHARP (fill))
-	fill = SCM_MAKINUM ((char) SCM_CHAR (fill));
-      SCM_ASRTGO (SCM_INUMP (fill)
-		  && -128 <= SCM_INUM (fill) && SCM_INUM (fill) < 128,
-		  badarg2);
+	fill = SCM_MAKINUM ((signed char) SCM_CHAR (fill));
+      SCM_ASRTGO (SCM_INUMP (fill), badarg2);
+      SCM_ASSERT_RANGE (SCM_ARG2, fill,
+                        -128 <= SCM_INUM (fill) && SCM_INUM (fill) < 128);
       for (i = base; n--; i += inc)
-	SCM_CHARS (ra)[i] = SCM_INUM (fill);
+	((char *) SCM_UVECTOR_BASE (ra))[i] = SCM_INUM (fill);
       break;
     case scm_tc7_bvect:
       { /* scope */
 	long *ve = (long *) SCM_VELTS (ra);
-	if (1 == inc && (n >= SCM_LONG_BIT || n == SCM_LENGTH (ra)))
+	if (1 == inc && (n >= SCM_LONG_BIT || n == SCM_BITVECTOR_LENGTH (ra)))
 	  {
 	    i = base / SCM_LONG_BIT;
 	    if (SCM_FALSEP (fill))
@@ -523,7 +529,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore)
 		  ve[i] |= ~(~0L << ((base + n) % SCM_LONG_BIT));
 	      }
 	    else
-	      badarg2:SCM_WTA (2,fill);
+	      badarg2:SCM_WRONG_TYPE_ARG (2, fill);
 	  }
 	else
 	  {
@@ -540,7 +546,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore)
       }
     case scm_tc7_uvect:
       { /* scope */
-	unsigned long f = SCM_NUM2ULONG (2,fill);
+	unsigned long f = SCM_NUM2ULONG (2, fill);
 	unsigned long *ve = (unsigned long *) SCM_VELTS (ra);
 
 	for (i = base; n--; i += inc)
@@ -549,7 +555,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore)
       }
     case scm_tc7_ivect:
       { /* scope */
-	long f = SCM_NUM2LONG (2,fill);
+	long f = SCM_NUM2LONG (2, fill);
 	long *ve = (long *) SCM_VELTS (ra);
 
 	for (i = base; n--; i += inc)
@@ -571,7 +577,7 @@ scm_array_fill_int (SCM ra, SCM fill, SCM ignore)
 #ifdef HAVE_LONG_LONGS
     case scm_tc7_llvect:
       { /* scope */
-	long long f = SCM_NUM2LONG_LONG (2,fill);
+	long long f = SCM_NUM2LONG_LONG (2, fill);
 	long long *ve = (long long *) SCM_VELTS (ra);
 
 	for (i = base; n--; i += inc)
@@ -628,23 +634,14 @@ racp (SCM src, SCM dst)
 {
   long n = (SCM_ARRAY_DIMS (src)->ubnd - SCM_ARRAY_DIMS (src)->lbnd + 1);
   long inc_d, inc_s = SCM_ARRAY_DIMS (src)->inc;
-  scm_sizet i_d, i_s = SCM_ARRAY_BASE (src);
+  unsigned long i_d, i_s = SCM_ARRAY_BASE (src);
   dst = SCM_CAR (dst);
   inc_d = SCM_ARRAY_DIMS (dst)->inc;
   i_d = SCM_ARRAY_BASE (dst);
   src = SCM_ARRAY_V (src);
   dst = SCM_ARRAY_V (dst);
 
-
-  /* untested optimization: don't copy if we're we. This allows the
-     ugly UNICOS macros (IVDEP) to go .     
-   */
-     
-  if (SCM_EQ_P (src, dst))
-    return 1 ;
-  
-  switch SCM_TYP7
-    (dst)
+  switch SCM_TYP7 (dst)
     {
     default:
     gencase:
@@ -652,17 +649,25 @@ racp (SCM src, SCM dst)
     case scm_tc7_wvect:
 
       for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	scm_array_set_x (dst, scm_cvref (src, i_s, SCM_UNDEFINED), SCM_MAKINUM (i_d));
+	scm_array_set_x (dst,
+			 scm_cvref (src, i_s, SCM_UNDEFINED),
+			 SCM_MAKINUM (i_d));
       break;
     case scm_tc7_string:
-    case scm_tc7_byvect:
-      if (scm_tc7_string != SCM_TYP7 (dst))
+      if (SCM_TYP7 (src) != scm_tc7_string)
 	goto gencase;
       for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	SCM_CHARS (dst)[i_d] = SCM_CHARS (src)[i_s];
+	SCM_STRING_CHARS (dst)[i_d] = SCM_STRING_CHARS (src)[i_s];
+      break;
+    case scm_tc7_byvect:
+      if (SCM_TYP7 (src) != scm_tc7_byvect)
+	goto gencase;
+      for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+	((char *) SCM_UVECTOR_BASE (dst))[i_d]
+	  = ((char *) SCM_UVECTOR_BASE (src))[i_s];
       break;
     case scm_tc7_bvect:
-      if (scm_tc7_bvect != SCM_TYP7 (dst))
+      if (SCM_TYP7 (src) != scm_tc7_bvect)
 	goto gencase;
       if (1 == inc_d && 1 == inc_s && i_s % SCM_LONG_BIT == i_d % SCM_LONG_BIT && n >= SCM_LONG_BIT)
 	{
@@ -677,8 +682,9 @@ racp (SCM src, SCM dst)
 	      sv++;
 	      n -= SCM_LONG_BIT - (i_s % SCM_LONG_BIT);
 	    }
+	  IVDEP (src != dst,
 		 for (; n >= SCM_LONG_BIT; n -= SCM_LONG_BIT, sv++, dv++)
-		 * dv = *sv;
+		 *dv = *sv;)
 	    if (n)		/* trailing partial word */
 	      *dv = (*dv & (~0L << n)) | (*sv & ~(~0L << n));
 	}
@@ -697,8 +703,9 @@ racp (SCM src, SCM dst)
       else
 	{
 	  long *d = (long *) SCM_VELTS (dst), *s = (long *) SCM_VELTS (src);
+	  IVDEP (src != dst,
 		 for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-		 d[i_d] = s[i_s];
+		 d[i_d] = s[i_s];)
 	    break;
 	}
     case scm_tc7_ivect:
@@ -707,9 +714,10 @@ racp (SCM src, SCM dst)
       else
 	{
 	  long *d = (long *) SCM_VELTS (dst), *s = (long *) SCM_VELTS (src);
-	  for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-            d[i_d] = s[i_s];
-          break;
+	  IVDEP (src != dst,
+		 for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		 d[i_d] = s[i_s];)
+	    break;
 	}
     case scm_tc7_fvect:
       {
@@ -722,17 +730,20 @@ racp (SCM src, SCM dst)
 	    goto gencase;
 	  case scm_tc7_ivect:
 	  case scm_tc7_uvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-		   d[i_d] = ((long *) s)[i_s];
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		   d[i_d] = ((long *) s)[i_s];)
 	      break;
 	  case scm_tc7_fvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-		   d[i_d] = s[i_s];
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		   d[i_d] = s[i_s];)
 	      break;
 	  case scm_tc7_dvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	      d[i_d] = ((double *) s)[i_s];
-	    break;
+	    IVDEP (src !=dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		   d[i_d] = ((double *) s)[i_s];)
+	      break;
 	  }
 	break;
       }
@@ -747,16 +758,19 @@ racp (SCM src, SCM dst)
 	    goto gencase;
 	  case scm_tc7_ivect:
 	  case scm_tc7_uvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-		   d[i_d] = ((long *) s)[i_s];
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		   d[i_d] = ((long *) s)[i_s];)
 	      break;
 	  case scm_tc7_fvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-		   d[i_d] = ((float *) s)[i_s];
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		   d[i_d] = ((float *) s)[i_s];)
 	      break;
 	  case scm_tc7_dvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-		   d[i_d] = s[i_s];
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+		   d[i_d] = s[i_s];)
 	      break;
 	  }
 	break;
@@ -772,33 +786,37 @@ racp (SCM src, SCM dst)
 	    goto gencase;
 	  case scm_tc7_ivect:
 	  case scm_tc7_uvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	      {
-		d[i_d][0] = ((long *) s)[i_s];
-		d[i_d][1] = 0.0;
-	      }
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+	           {
+		     d[i_d][0] = ((long *) s)[i_s];
+		     d[i_d][1] = 0.0;
+		   })
 	      break;
 	  case scm_tc7_fvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	      {
-		d[i_d][0] = ((float *) s)[i_s];
-		d[i_d][1] = 0.0;
-	      }
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+	           {
+		     d[i_d][0] = ((float *) s)[i_s];
+		     d[i_d][1] = 0.0;
+		   })
 	      break;
 	  case scm_tc7_dvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	      {
-		d[i_d][0] = ((double *) s)[i_s];
-		d[i_d][1] = 0.0;
-	      }
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+	           {
+		     d[i_d][0] = ((double *) s)[i_s];
+		     d[i_d][1] = 0.0;
+		   })
 	      break;
 	  case scm_tc7_cvect:
-	    for (; n-- > 0; i_s += inc_s, i_d += inc_d)
-	      {
-		d[i_d][0] = s[i_s][0];
-		d[i_d][1] = s[i_s][1];
+	    IVDEP (src != dst,
+		   for (; n-- > 0; i_s += inc_s, i_d += inc_d)
+	           {
+		     d[i_d][0] = s[i_s][0];
+		     d[i_d][1] = s[i_s][1];
+		   })
 	      }
-	  }
 	break;
       }
     }
@@ -811,7 +829,8 @@ SCM_REGISTER_PROC(s_array_copy_in_order_x, "array-copy-in-order!", 2, 0, 0, scm_
 
 SCM_DEFINE (scm_array_copy_x, "array-copy!", 2, 0, 0,
 	    (SCM src, SCM dst),
-	    "Copies every element from vector or array @var{source} to the\n"
+	    "@deffnx {Scheme Procedure} array-copy-in-order! src dst\n"
+	    "Copy every element from vector or array @var{source} to the\n"
 	    "corresponding element of @var{destination}.  @var{destination} must have\n"
 	    "the same rank as @var{source}, and be at least as large in each\n"
 	    "dimension.  The order is unspecified.")
@@ -830,7 +849,7 @@ scm_ra_eqp (SCM ra0, SCM ras)
 {
   SCM ra1 = SCM_CAR (ras), ra2 = SCM_CAR (SCM_CDR (ras));
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   long inc2 = SCM_ARRAY_DIMS (ra1)->inc;
@@ -889,7 +908,7 @@ static int
 ra_compare (SCM ra0,SCM ra1,SCM ra2,int opt)
 {
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   long inc2 = SCM_ARRAY_DIMS (ra1)->inc;
@@ -983,13 +1002,13 @@ int
 scm_ra_sum (SCM ra0, SCM ras)
 {
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NNULLP(ras))
     {
       SCM ra1 = SCM_CAR (ras);
-      scm_sizet i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       switch (SCM_TYP7 (ra0) == SCM_TYP7 (ra1) ? SCM_TYP7 (ra0) : 0)
@@ -1022,7 +1041,7 @@ int
 scm_ra_difference (SCM ra0, SCM ras)
 {
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
@@ -1049,7 +1068,7 @@ scm_ra_difference (SCM ra0, SCM ras)
   else
     {
       SCM ra1 = SCM_CAR (ras);
-      scm_sizet i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       switch (SCM_TYP7 (ra0) == SCM_TYP7 (ra1) ? SCM_TYP7 (ra0) : 0)
@@ -1078,13 +1097,13 @@ int
 scm_ra_product (SCM ra0, SCM ras)
 {
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NNULLP (ras))
     {
       SCM ra1 = SCM_CAR (ras);
-      scm_sizet i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       switch (SCM_TYP7 (ra0) == SCM_TYP7 (ra1) ? SCM_TYP7 (ra0) : 0)
@@ -1129,7 +1148,7 @@ int
 scm_ra_divide (SCM ra0, SCM ras)
 {
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
@@ -1164,7 +1183,7 @@ scm_ra_divide (SCM ra0, SCM ras)
   else
     {
       SCM ra1 = SCM_CAR (ras);
-      scm_sizet i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       switch (SCM_TYP7 (ra0) == SCM_TYP7 (ra1) ? SCM_TYP7 (ra0) : 0)
@@ -1220,12 +1239,12 @@ ramap (SCM ra0,SCM proc,SCM ras)
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
     for (; i <= n; i++)
-      scm_array_set_x (ra0, scm_apply (proc, SCM_EOL, SCM_EOL), SCM_MAKINUM (i * inc + base));
+      scm_array_set_x (ra0, scm_call_0 (proc), SCM_MAKINUM (i * inc + base));
   else
     {
       SCM ra1 = SCM_CAR (ras);
       SCM args, *ve = &ras;
-      scm_sizet k, i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long k, i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       ras = SCM_CDR (ras);
@@ -1239,10 +1258,10 @@ ramap (SCM ra0,SCM proc,SCM ras)
       for (; i <= n; i++, i1 += inc1)
 	{
 	  args = SCM_EOL;
-	  for (k = SCM_LENGTH (ras); k--;)
+	  for (k = SCM_INUM (scm_uniform_vector_length (ras)); k--;)
 	    args = scm_cons (scm_uniform_vector_ref (ve[k], SCM_MAKINUM (i)), args);
 	  args = scm_cons (scm_cvref (ra1, i1, SCM_UNDEFINED), args);
-	  scm_array_set_x (ra0, scm_apply (proc, args, SCM_EOL), SCM_MAKINUM (i * inc + base));
+	  scm_array_set_x (ra0, scm_apply_0 (proc, args), SCM_MAKINUM (i * inc + base));
 	}
     }
   return 1;
@@ -1254,7 +1273,7 @@ ramap_cxr (SCM ra0,SCM proc,SCM ras)
 {
   SCM ra1 = SCM_CAR (ras);
   SCM e1 = SCM_UNDEFINED;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc, inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra1)->lbnd + 1;
   ra0 = SCM_ARRAY_V (ra0);
@@ -1264,7 +1283,7 @@ ramap_cxr (SCM ra0,SCM proc,SCM ras)
     default:
     gencase:
  for (; n-- > 0; i0 += inc0, i1 += inc1)
-   scm_array_set_x (ra0, scm_apply (proc, RVREF (ra1, i1, e1), scm_listofnull), SCM_MAKINUM (i0));
+   scm_array_set_x (ra0, scm_call_1 (proc, RVREF (ra1, i1, e1)), SCM_MAKINUM (i0));
  break;
     case scm_tc7_fvect:
       {
@@ -1316,7 +1335,7 @@ ramap_rp (SCM ra0,SCM proc,SCM ras)
   SCM ra1 = SCM_CAR (ras), ra2 = SCM_CAR (SCM_CDR (ras));
   SCM e1 = SCM_UNDEFINED, e2 = SCM_UNDEFINED;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1), i2 = SCM_ARRAY_BASE (ra2);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   long inc2 = SCM_ARRAY_DIMS (ra1)->inc;
@@ -1342,7 +1361,7 @@ ramap_rp (SCM ra0,SCM proc,SCM ras)
 	     */
 	    SCM n1 = SCM_MAKINUM (((long *) SCM_UNPACK (SCM_CDR (ra1)))[i1]);
 	    SCM n2 = SCM_MAKINUM (((long *) SCM_UNPACK (SCM_CDR (ra2)))[i2]);
-	    if (SCM_FALSEP (SCM_SUBRF (proc) (n1, n2)));
+	    if (SCM_FALSEP (SCM_SUBRF (proc) (n1, n2)))
 	      SCM_BITVEC_CLR (ra0, i0);
 	  }
       break;
@@ -1396,21 +1415,27 @@ ramap_rp (SCM ra0,SCM proc,SCM ras)
 
 
 static int
-ramap_1 (SCM ra0,SCM proc,SCM ras)
+ramap_1 (SCM ra0, SCM proc, SCM ras)
 {
   SCM ra1 = SCM_CAR (ras);
   SCM e1 = SCM_UNDEFINED;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1);
-  long inc0 = SCM_ARRAY_DIMS (ra0)->inc, inc1 = SCM_ARRAY_DIMS (ra1)->inc;
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i1 = SCM_ARRAY_BASE (ra1);
+  long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
+  long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   ra0 = SCM_ARRAY_V (ra0);
-  ra1 = SCM_ARRAY_V (ra1);
+  ra1 = SCM_ARRAY_V (ra1);  
   if (scm_tc7_vector == SCM_TYP7 (ra0) || scm_tc7_wvect == SCM_TYP7 (ra0))
     for (; n-- > 0; i0 += inc0, i1 += inc1)
-      scm_array_set_x (ra0, SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED)), SCM_MAKINUM (i0));
+      scm_array_set_x (ra0,
+                       SCM_SUBRF (proc) (scm_cvref (ra1, i1, SCM_UNDEFINED)),
+                       SCM_MAKINUM (i0));
   else
     for (; n-- > 0; i0 += inc0, i1 += inc1)
-      scm_array_set_x (ra0, SCM_SUBRF (proc) (RVREF (ra1, i1, e1)), SCM_MAKINUM (i0));
+      scm_array_set_x (ra0,
+                       SCM_SUBRF (proc) (RVREF (ra1, i1, e1)),
+                       SCM_MAKINUM (i0));
   return 1;
 }
 
@@ -1422,7 +1447,7 @@ ramap_2o (SCM ra0,SCM proc,SCM ras)
   SCM ra1 = SCM_CAR (ras);
   SCM e1 = SCM_UNDEFINED;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0), i1 = SCM_ARRAY_BASE (ra1);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc, inc1 = SCM_ARRAY_DIMS (ra1)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   ra1 = SCM_ARRAY_V (ra1);
@@ -1444,7 +1469,7 @@ ramap_2o (SCM ra0,SCM proc,SCM ras)
     {
       SCM ra2 = SCM_CAR (ras);
       SCM e2 = SCM_UNDEFINED;
-      scm_sizet i2 = SCM_ARRAY_BASE (ra2);
+      unsigned long i2 = SCM_ARRAY_BASE (ra2);
       long inc2 = SCM_ARRAY_DIMS (ra2)->inc;
       ra2 = SCM_ARRAY_V (ra2);
       if (scm_tc7_vector == SCM_TYP7 (ra0) || scm_tc7_wvect == SCM_TYP7 (ra0))
@@ -1468,7 +1493,7 @@ ramap_a (SCM ra0,SCM proc,SCM ras)
 {
   SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd - SCM_ARRAY_DIMS (ra0)->lbnd + 1;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
@@ -1477,7 +1502,7 @@ ramap_a (SCM ra0,SCM proc,SCM ras)
   else
     {
       SCM ra1 = SCM_CAR (ras);
-      scm_sizet i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       for (; n-- > 0; i0 += inc0, i1 += inc1)
@@ -1492,85 +1517,96 @@ SCM_REGISTER_PROC(s_array_map_in_order_x, "array-map-in-order!", 2, 0, 1, scm_ar
 
 
 SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
-	    (SCM ra0, SCM proc, SCM lra),
-	    "@var{array1}, @dots{} must have the same number of dimensions as\n"
-	    "@var{array0} and have a range for each index which includes the range\n"
-	    "for the corresponding index in @var{array0}.  @var{proc} is applied to\n"
-	    "each tuple of elements of @var{array1} @dots{} and the result is stored\n"
-	    "as the corresponding element in @var{array0}.  The value returned is\n"
-	    "unspecified.  The order of application is unspecified.")
+	    (SCM dest, SCM proc, SCM sources),
+	    "@var{dest} must be an array, and @var{sources} must be a non-empty list\n"
+            "of arrays where each element must have the same number of dimensions as\n"
+            "@var{dest} and must have a range for each index which includes the range\n"
+	    "for the corresponding index in @var{dest}.  @var{proc} is applied to\n"
+	    "each tuple of elements of @var{sources} and the result is stored as the\n"
+            "corresponding element in @var{dest}.  The value returned is unspecified, and\n"
+            "the order of application is unspecified.\n")
 #define FUNC_NAME s_scm_array_map_x
 {
   SCM_VALIDATE_PROC (2,proc);
-  SCM_VALIDATE_REST_ARGUMENT (lra);
+
   switch (SCM_TYP7 (proc))
     {
     default:
     gencase:
- scm_ramapc (ramap, proc, ra0, lra, FUNC_NAME);
- return SCM_UNSPECIFIED;
+      scm_ramapc (ramap, proc, dest, sources, FUNC_NAME);
+      return SCM_UNSPECIFIED;
     case scm_tc7_subr_1:
-      scm_ramapc (ramap_1, proc, ra0, lra, FUNC_NAME);
+      if (! SCM_CONSP (sources))
+        SCM_WRONG_NUM_ARGS ();  /* need 1 source */
+      scm_ramapc (ramap_1, proc, dest, sources, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_subr_2:
+      if (! (SCM_CONSP (sources) && SCM_CONSP (SCM_CDR (sources))))
+        SCM_WRONG_NUM_ARGS ();  /* need 2 sources */
+      goto subr_2o;
     case scm_tc7_subr_2o:
-      scm_ramapc (ramap_2o, proc, ra0, lra, FUNC_NAME);
+      if (! SCM_CONSP (sources))
+        SCM_WRONG_NUM_ARGS ();  /* need 1 source */
+    subr_2o:
+      scm_ramapc (ramap_2o, proc, dest, sources, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_cxr:
       if (!SCM_SUBRF (proc))
 	goto gencase;
-      scm_ramapc (ramap_cxr, proc, ra0, lra, FUNC_NAME);
+      if (! SCM_CONSP (sources))
+        SCM_WRONG_NUM_ARGS ();  /* need 1 source */
+      scm_ramapc (ramap_cxr, proc, dest, sources, FUNC_NAME);
       return SCM_UNSPECIFIED;
     case scm_tc7_rpsubr:
       {
 	ra_iproc *p;
-	if (SCM_FALSEP (scm_array_p (ra0, SCM_BOOL_T)))
+	if (SCM_FALSEP (scm_array_p (dest, SCM_BOOL_T)))
 	  goto gencase;
-	scm_array_fill_x (ra0, SCM_BOOL_T);
+	scm_array_fill_x (dest, SCM_BOOL_T);
 	for (p = ra_rpsubrs; p->name; p++)
 	  if (SCM_EQ_P (proc, p->sproc))
 	    {
-	      while (SCM_NNULLP (lra) && SCM_NNULLP (SCM_CDR (lra)))
+	      while (SCM_NNULLP (sources) && SCM_NNULLP (SCM_CDR (sources)))
 		{
-		  scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, FUNC_NAME);
-		  lra = SCM_CDR (lra);
+		  scm_ramapc (p->vproc, SCM_UNDEFINED, dest, sources, FUNC_NAME);
+		  sources = SCM_CDR (sources);
 		}
 	      return SCM_UNSPECIFIED;
 	    }
-	while (SCM_NNULLP (lra) && SCM_NNULLP (SCM_CDR (lra)))
+	while (SCM_NNULLP (sources) && SCM_NNULLP (SCM_CDR (sources)))
 	  {
-	    scm_ramapc (ramap_rp, proc, ra0, lra, FUNC_NAME);
-	    lra = SCM_CDR (lra);
+	    scm_ramapc (ramap_rp, proc, dest, sources, FUNC_NAME);
+	    sources = SCM_CDR (sources);
 	  }
 	return SCM_UNSPECIFIED;
       }
     case scm_tc7_asubr:
-      if (SCM_NULLP (lra))
+      if (SCM_NULLP (sources))
 	{
 	  SCM prot, fill = SCM_SUBRF (proc) (SCM_UNDEFINED, SCM_UNDEFINED);
 	  if (SCM_INUMP(fill))
 	    {
-	      prot = scm_array_prototype (ra0);
+	      prot = scm_array_prototype (dest);
 	      if (SCM_INEXACTP (prot))
 		fill = scm_make_real ((double) SCM_INUM (fill));
 	    }
 
-	  scm_array_fill_x (ra0, fill);
+	  scm_array_fill_x (dest, fill);
 	}
       else
 	{
-	  SCM tail, ra1 = SCM_CAR (lra);
-	  SCM v0 = (SCM_ARRAYP (ra0) ? SCM_ARRAY_V (ra0) : ra0);
+	  SCM tail, ra1 = SCM_CAR (sources);
+	  SCM v0 = (SCM_ARRAYP (dest) ? SCM_ARRAY_V (dest) : dest);
 	  ra_iproc *p;
 	  /* Check to see if order might matter.
 	     This might be an argument for a separate
 	     SERIAL-ARRAY-MAP! */
 	  if (SCM_EQ_P (v0, ra1) 
 	      || (SCM_ARRAYP (ra1) && SCM_EQ_P (v0, SCM_ARRAY_V (ra1))))
-	    if (!SCM_EQ_P (ra0, ra1) 
-		|| (SCM_ARRAYP(ra0) && !SCM_ARRAY_CONTP(ra0)))
+	    if (!SCM_EQ_P (dest, ra1) 
+		|| (SCM_ARRAYP(dest) && !SCM_ARRAY_CONTP(dest)))
 	      goto gencase;
-	  for (tail = SCM_CDR (lra); SCM_NNULLP (tail); tail = SCM_CDR (tail))
+	  for (tail = SCM_CDR (sources); SCM_NNULLP (tail); tail = SCM_CDR (tail))
 	    {
 	      ra1 = SCM_CAR (tail);
 	      if (SCM_EQ_P (v0, ra1) 
@@ -1580,22 +1616,22 @@ SCM_DEFINE (scm_array_map_x, "array-map!", 2, 0, 1,
 	  for (p = ra_asubrs; p->name; p++)
 	    if (SCM_EQ_P (proc, p->sproc))
 	      {
-		if (!SCM_EQ_P (ra0, SCM_CAR (lra)))
-		  scm_ramapc (scm_array_identity, SCM_UNDEFINED, ra0, scm_cons (SCM_CAR (lra), SCM_EOL), FUNC_NAME);
-		lra = SCM_CDR (lra);
+		if (!SCM_EQ_P (dest, SCM_CAR (sources)))
+		  scm_ramapc (scm_array_identity, SCM_UNDEFINED, dest, scm_cons (SCM_CAR (sources), SCM_EOL), FUNC_NAME);
+		sources = SCM_CDR (sources);
 		while (1)
 		  {
-		    scm_ramapc (p->vproc, SCM_UNDEFINED, ra0, lra, FUNC_NAME);
-		    if (SCM_IMP (lra) || SCM_IMP (SCM_CDR (lra)))
+		    scm_ramapc (p->vproc, SCM_UNDEFINED, dest, sources, FUNC_NAME);
+		    if (SCM_IMP (sources) || SCM_IMP (SCM_CDR (sources)))
 		      return SCM_UNSPECIFIED;
-		    lra = SCM_CDR (lra);
+		    sources = SCM_CDR (sources);
 		  }
 	      }
-	  scm_ramapc (ramap_2o, proc, ra0, lra, FUNC_NAME);
-	  lra = SCM_CDR (lra);
-	  if (SCM_NIMP (lra))
-	    for (lra = SCM_CDR (lra); SCM_NIMP (lra); lra = SCM_CDR (lra))
-	      scm_ramapc (ramap_a, proc, ra0, lra, FUNC_NAME);
+	  scm_ramapc (ramap_2o, proc, dest, sources, FUNC_NAME);
+	  sources = SCM_CDR (sources);
+	  if (SCM_NIMP (sources))
+	    for (sources = SCM_CDR (sources); SCM_NIMP (sources); sources = SCM_CDR (sources))
+	      scm_ramapc (ramap_a, proc, dest, sources, FUNC_NAME);
 	}
       return SCM_UNSPECIFIED;
     }
@@ -1607,18 +1643,18 @@ static int
 rafe (SCM ra0,SCM proc,SCM ras)
 {
   long i = SCM_ARRAY_DIMS (ra0)->lbnd;
-  scm_sizet i0 = SCM_ARRAY_BASE (ra0);
+  unsigned long i0 = SCM_ARRAY_BASE (ra0);
   long inc0 = SCM_ARRAY_DIMS (ra0)->inc;
   long n = SCM_ARRAY_DIMS (ra0)->ubnd;
   ra0 = SCM_ARRAY_V (ra0);
   if (SCM_NULLP (ras))
     for (; i <= n; i++, i0 += inc0)
-      scm_apply (proc, scm_cvref (ra0, i0, SCM_UNDEFINED), scm_listofnull);
+      scm_call_1 (proc, scm_cvref (ra0, i0, SCM_UNDEFINED));
   else
     {
       SCM ra1 = SCM_CAR (ras);
       SCM args, *ve = &ras;
-      scm_sizet k, i1 = SCM_ARRAY_BASE (ra1);
+      unsigned long k, i1 = SCM_ARRAY_BASE (ra1);
       long inc1 = SCM_ARRAY_DIMS (ra1)->inc;
       ra1 = SCM_ARRAY_V (ra1);
       ras = SCM_CDR (ras);
@@ -1632,10 +1668,10 @@ rafe (SCM ra0,SCM proc,SCM ras)
       for (; i <= n; i++, i0 += inc0, i1 += inc1)
 	{
 	  args = SCM_EOL;
-	  for (k = SCM_LENGTH (ras); k--;)
+	  for (k = SCM_INUM (scm_uniform_vector_length (ras)); k--;)
 	    args = scm_cons (scm_uniform_vector_ref (ve[k], SCM_MAKINUM (i)), args);
 	  args = scm_cons2 (scm_cvref (ra0, i0, SCM_UNDEFINED), scm_cvref (ra1, i1, SCM_UNDEFINED), args);
-	  scm_apply (proc, args, SCM_EOL);
+	  scm_apply_0 (proc, args);
 	}
     }
   return 1;
@@ -1644,7 +1680,7 @@ rafe (SCM ra0,SCM proc,SCM ras)
 
 SCM_DEFINE (scm_array_for_each, "array-for-each", 2, 0, 1,
 	    (SCM proc, SCM ra0, SCM lra),
-	    "@var{proc} is applied to each tuple of elements of @var{array0} @dots{}\n"
+	    "Apply @var{proc} to each tuple of elements of @var{array0} @dots{}\n"
 	    "in row-major order.  The value returned is unspecified.")
 #define FUNC_NAME s_scm_array_for_each
 {
@@ -1657,38 +1693,38 @@ SCM_DEFINE (scm_array_for_each, "array-for-each", 2, 0, 1,
 
 SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 	    (SCM ra, SCM proc),
-	    "applies @var{proc} to the indices of each element of @var{array} in\n"
+	    "Apply @var{proc} to the indices of each element of @var{array} in\n"
 	    "turn, storing the result in the corresponding element.  The value\n"
 	    "returned and the order of application are unspecified.\n\n"
 	    "One can implement @var{array-indexes} as\n"
-	    "@example\n"
+	    "@lisp\n"
 	    "(define (array-indexes array)\n"
 	    "    (let ((ra (apply make-array #f (array-shape array))))\n"
 	    "      (array-index-map! ra (lambda x x))\n"
 	    "      ra))\n"
-	    "@end example\n"
+	    "@end lisp\n"
 	    "Another example:\n"
-	    "@example\n"
+	    "@lisp\n"
 	    "(define (apl:index-generator n)\n"
 	    "    (let ((v (make-uniform-vector n 1)))\n"
 	    "      (array-index-map! v (lambda (i) i))\n"
 	    "      v))\n"
-	    "@end example")
+	    "@end lisp")
 #define FUNC_NAME s_scm_array_index_map_x
 {
-  scm_sizet i;
+  unsigned long i;
   SCM_VALIDATE_NIM (1,ra);
   SCM_VALIDATE_PROC (2,proc);
   switch (SCM_TYP7(ra))
     {
     default:
-    badarg:SCM_WTA (1,ra);
+    badarg:SCM_WRONG_TYPE_ARG (1, ra);
     case scm_tc7_vector:
     case scm_tc7_wvect:
       {
 	SCM *ve = SCM_VELTS (ra);
-	for (i = 0; i < SCM_LENGTH (ra); i++)
-	  ve[i] = scm_apply (proc, SCM_MAKINUM (i), scm_listofnull);
+	for (i = 0; i < SCM_VECTOR_LENGTH (ra); i++)
+	  ve[i] = scm_call_1 (proc, SCM_MAKINUM (i));
 	return SCM_UNSPECIFIED;
       }
     case scm_tc7_string:
@@ -1703,10 +1739,13 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
     case scm_tc7_fvect:
     case scm_tc7_dvect:
     case scm_tc7_cvect:
-      for (i = 0; i < SCM_LENGTH (ra); i++)
-	scm_array_set_x (ra, scm_apply (proc, SCM_MAKINUM (i), scm_listofnull),
-			 SCM_MAKINUM (i));
-      return SCM_UNSPECIFIED;
+      {
+	unsigned long int length = SCM_INUM (scm_uniform_vector_length (ra));
+	for (i = 0; i < length; i++)
+	  scm_array_set_x (ra, scm_call_1 (proc, SCM_MAKINUM (i)),
+			   SCM_MAKINUM (i));
+	return SCM_UNSPECIFIED;
+      }
     case scm_tc7_smob:
       SCM_ASRTGO (SCM_ARRAYP (ra), badarg);
       {
@@ -1715,8 +1754,7 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 	long *vinds = (long *) SCM_VELTS (inds);
 	int j, k, kmax = SCM_ARRAY_NDIM (ra) - 1;
 	if (kmax < 0)
-	  return scm_array_set_x (ra, scm_apply(proc, SCM_EOL, SCM_EOL),
-				  SCM_EOL);
+	  return scm_array_set_x (ra, scm_call_0 (proc), SCM_EOL);
 	for (k = 0; k <= kmax; k++)
 	  vinds[k] = SCM_ARRAY_DIMS (ra)[k].lbnd;
 	k = kmax;
@@ -1731,7 +1769,7 @@ SCM_DEFINE (scm_array_index_map_x, "array-index-map!", 2, 0, 0,
 		    for (j = kmax + 1, args = SCM_EOL; j--;)
 		      args = scm_cons (SCM_MAKINUM (vinds[j]), args);
 		    scm_array_set_x (SCM_ARRAY_V (ra),
-				     scm_apply (proc, args, SCM_EOL),
+				     scm_apply_0 (proc, args),
 				     SCM_MAKINUM (i));
 		    i += SCM_ARRAY_DIMS (ra)[k].inc;
 		  }
@@ -1759,9 +1797,9 @@ static int
 raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
 {
   SCM e0 = SCM_UNDEFINED, e1 = SCM_UNDEFINED;
-  scm_sizet i0 = 0, i1 = 0;
+  unsigned long i0 = 0, i1 = 0;
   long inc0 = 1, inc1 = 1;
-  scm_sizet n = SCM_LENGTH (ra0);
+  unsigned long n;
   ra1 = SCM_CAR (ra1);
   if (SCM_ARRAYP(ra0))
     {
@@ -1770,6 +1808,8 @@ raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
       inc0 = SCM_ARRAY_DIMS (ra0)->inc;
       ra0 = SCM_ARRAY_V (ra0);
     }
+  else
+    n = SCM_INUM (scm_uniform_vector_length (ra0));
   if (SCM_ARRAYP (ra1))
     {
       i1 = SCM_ARRAY_BASE (ra1);
@@ -1793,10 +1833,18 @@ raeql_1 (SCM ra0,SCM as_equal,SCM ra1)
 	}
       return 1;
     case scm_tc7_string:
+      {
+	char *v0 = SCM_STRING_CHARS (ra0) + i0;
+	char *v1 = SCM_STRING_CHARS (ra1) + i1;
+	for (; n--; v0 += inc0, v1 += inc1)
+	  if (*v0 != *v1)
+	    return 0;
+	return 1;
+      }
     case scm_tc7_byvect:
       {
-	char *v0 = SCM_CHARS (ra0) + i0;
-	char *v1 = SCM_CHARS (ra1) + i1;
+	char *v0 = ((char *) SCM_UVECTOR_BASE (ra0)) + i0;
+	char *v1 = ((char *) SCM_UVECTOR_BASE (ra1)) + i1;
 	for (; n--; v0 += inc0, v1 += inc1)
 	  if (*v0 != *v1)
 	    return 0;
@@ -1877,9 +1925,9 @@ static int
 raeql (SCM ra0,SCM as_equal,SCM ra1)
 {
   SCM v0 = ra0, v1 = ra1;
-  scm_array_dim dim0, dim1;
-  scm_array_dim *s0 = &dim0, *s1 = &dim1;
-  scm_sizet bas0 = 0, bas1 = 0;
+  scm_t_array_dim dim0, dim1;
+  scm_t_array_dim *s0 = &dim0, *s1 = &dim1;
+  unsigned long bas0 = 0, bas1 = 0;
   int k, unroll = 1, vlen = 1, ndim = 1;
   if (SCM_ARRAYP (ra0))
     {
@@ -1892,7 +1940,7 @@ raeql (SCM ra0,SCM as_equal,SCM ra1)
     {
       s0->inc = 1;
       s0->lbnd = 0;
-      s0->ubnd = SCM_LENGTH (v0) - 1;
+      s0->ubnd = SCM_INUM (scm_uniform_vector_length (v0)) - 1;
       unroll = 0;
     }
   if (SCM_ARRAYP (ra1))
@@ -1912,7 +1960,7 @@ raeql (SCM ra0,SCM as_equal,SCM ra1)
 	return 0;
       s1->inc = 1;
       s1->lbnd = 0;
-      s1->ubnd = SCM_LENGTH (v1) - 1;
+      s1->ubnd = SCM_INUM (scm_uniform_vector_length (v1)) - 1;
       unroll = 0;
     }
   if (SCM_TYP7 (v0) != SCM_TYP7 (v1))
@@ -1943,11 +1991,12 @@ scm_raequal (SCM ra0, SCM ra1)
 /* GJB:FIXME:: Why not use SCM_DEFINE1 for array-equal? */
 SCM_DEFINE1 (scm_array_equal_p, "array-equal?", scm_tc7_rpsubr,
 	     (SCM ra0, SCM ra1),
-	     "Returns @code{#t} iff all arguments are arrays with the same shape, the\n"
-	     "same type, and have corresponding elements which are either\n"
-	     "@code{equal?}  or @code{array-equal?}.  This function differs from\n"
-	     "@code{equal?} in that a one dimensional shared array may be\n"
-	     "@var{array-equal?} but not @var{equal?} to a vector or uniform vector.")
+	    "Return @code{#t} iff all arguments are arrays with the same\n"
+	    "shape, the same type, and have corresponding elements which are\n"
+	    "either @code{equal?}  or @code{array-equal?}.  This function\n"
+	    "differs from @code{equal?} in that a one dimensional shared\n"
+	    "array may be @var{array-equal?} but not @var{equal?} to a\n"
+	    "vector or uniform vector.")
 #define FUNC_NAME s_scm_array_equal_p
 {
 }
@@ -1971,6 +2020,7 @@ scm_array_equal_p (SCM ra0, SCM ra1)
     case scm_tc7_byvect:
     case scm_tc7_uvect:
     case scm_tc7_ivect:
+    case scm_tc7_svect:
     case scm_tc7_fvect:
     case scm_tc7_dvect:
     case scm_tc7_cvect:
@@ -1990,6 +2040,7 @@ scm_array_equal_p (SCM ra0, SCM ra1)
     case scm_tc7_byvect:
     case scm_tc7_uvect:
     case scm_tc7_ivect:
+    case scm_tc7_svect:
     case scm_tc7_fvect:
     case scm_tc7_dvect:
     case scm_tc7_cvect:
@@ -2004,12 +2055,19 @@ scm_array_equal_p (SCM ra0, SCM ra1)
 }
 
 
-
 static void
 init_raprocs (ra_iproc *subra)
 {
   for (; subra->name; subra++)
-    subra->sproc = SCM_CDR (scm_intern (subra->name, strlen (subra->name)));
+    {
+      SCM sym = scm_str2symbol (subra->name);
+      SCM var =
+	scm_sym2var (sym, scm_current_module_lookup_closure (), SCM_BOOL_F);
+      if (var != SCM_BOOL_F)
+	subra->sproc = SCM_VARIABLE_REF (var);
+      else
+	subra->sproc = SCM_BOOL_F;
+    }
 }
 
 
@@ -2018,8 +2076,8 @@ scm_init_ramap ()
 {
   init_raprocs (ra_rpsubrs);
   init_raprocs (ra_asubrs);
-  scm_make_subr (s_array_equal_p, scm_tc7_rpsubr, scm_array_equal_p);
-  scm_smobs[0x0ff & (scm_tc16_array >> 8)].equalp = scm_raequal;
+  scm_c_define_subr (s_array_equal_p, scm_tc7_rpsubr, scm_array_equal_p);
+  scm_smobs[SCM_TC2SMOBNUM (scm_tc16_array)].equalp = scm_raequal;
 #include "libguile/ramap.x"
   scm_add_feature (s_scm_array_for_each);
 }
