@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997,1998, 1999, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,1999,2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,7 +42,7 @@
 
 
 
-#include <stdio.h>
+#include <string.h>
 #include "libguile/_scm.h"
 #include "libguile/stackchk.h"
 #include "libguile/dynwind.h"
@@ -56,15 +56,9 @@
 #include "libguile/root.h"
 
 
-/* Define this if you want to try out the stack allocation of cwdr's
-   jumpbuf.  It works for me but I'm still worried that the dynwinds
-   might be able to make a mess. */
-
-#undef USE_STACKJMPBUF
-
 SCM scm_sys_protects[SCM_NUM_PROTECTS];
 
-long scm_tc16_root;
+scm_t_bits scm_tc16_root;
 
 #ifndef USE_THREADS
 struct scm_root_state *scm_root;
@@ -73,7 +67,7 @@ struct scm_root_state *scm_root;
 
 
 static SCM
-mark_root (SCM root)
+root_mark (SCM root)
 {
   scm_root_state *s = SCM_ROOT_STATE (root);
 
@@ -91,14 +85,12 @@ mark_root (SCM root)
   scm_gc_mark (s->def_errp);
   /* No need to gc mark def_loadp */
   scm_gc_mark (s->fluids);
-  scm_gc_mark (s->top_level_lookup_closure_var);
-  scm_gc_mark (s->system_transformer);
   return SCM_ROOT_STATE (root) -> parent;
 }
 
 
 static int
-print_root (SCM exp,SCM port,scm_print_state *pstate)
+root_print (SCM exp, SCM port, scm_print_state *pstate SCM_UNUSED)
 {
   scm_puts ("#<root ", port);
   scm_intprint(SCM_SEQ (SCM_ROOT_STATE (exp) -> rootcont), 16, port);
@@ -142,8 +134,6 @@ scm_make_root (SCM parent)
 	= root_state->def_errp
 	= root_state->cur_loadp
 	= root_state->fluids
-	= root_state->system_transformer
-	= root_state->top_level_lookup_closure_var
 	= root_state->handle
 	= root_state->parent
 	= SCM_BOOL_F;
@@ -179,7 +169,7 @@ scm_make_root (SCM parent)
 #if 0
 SCM scm_exitval;		/* INUM with return value */
 #endif
-static int n_dynamic_roots = 0;
+static long n_dynamic_roots = 0;
 
 
 /* cwdr fills out both of these structures, and then passes a pointer
@@ -246,13 +236,10 @@ cwdr_handler (void *data, SCM tag, SCM args)
  * in a messed up state.  */
 
 SCM 
-scm_internal_cwdr (scm_catch_body_t body, void *body_data,
-		   scm_catch_handler_t handler, void *handler_data,
+scm_internal_cwdr (scm_t_catch_body body, void *body_data,
+		   scm_t_catch_handler handler, void *handler_data,
 		   SCM_STACKITEM *stack_start)
 {
-#ifdef USE_STACKJMPBUF
-  scm_contregs static_contregs;
-#endif
   int old_ints_disabled = scm_ints_disabled;
   SCM old_rootcont, old_winds;
   struct cwdr_handler_data my_handler_data;
@@ -261,22 +248,22 @@ scm_internal_cwdr (scm_catch_body_t body, void *body_data,
   /* Create a fresh root continuation.  */
   {
     SCM new_rootcont;
-    SCM_NEWCELL (new_rootcont);
+
     SCM_REDEFER_INTS;
-#ifdef USE_STACKJMPBUF
-    SCM_SET_CONTREGS (new_rootcont, &static_contregs);
-#else
-    SCM_SET_CONTREGS (new_rootcont,
-		      scm_must_malloc (sizeof (scm_contregs),
-				       "inferior root continuation"));
-#endif
-    SCM_SET_CELL_TYPE (new_rootcont, scm_tc7_contin);
-    SCM_DYNENV (new_rootcont) = SCM_EOL;
-    SCM_BASE (new_rootcont) = stack_start;
-    SCM_SEQ (new_rootcont) = ++n_dynamic_roots;
+    {
+      scm_t_contregs *contregs = scm_must_malloc (sizeof (scm_t_contregs),
+						"inferior root continuation");
+
+      contregs->num_stack_items = 0;
+      contregs->dynenv = SCM_EOL;
+      contregs->base = stack_start;
+      contregs->seq = ++n_dynamic_roots;
+      contregs->throw_value = SCM_BOOL_F;
 #ifdef DEBUG_EXTENSIONS
-    SCM_DFRAME (new_rootcont) = 0;
+      contregs->dframe = 0;
 #endif
+      SCM_NEWSMOB (new_rootcont, scm_tc16_continuation, contregs);
+    }
     old_rootcont = scm_rootcont;
     scm_rootcont = new_rootcont;
     SCM_REALLOW_INTS;
@@ -300,9 +287,6 @@ scm_internal_cwdr (scm_catch_body_t body, void *body_data,
 
   scm_dowinds (old_winds, - scm_ilength (old_winds));
   SCM_REDEFER_INTS;
-#ifdef USE_STACKCJMPBUF
-  SCM_SET_CONTREGS (scm_rootcont, NULL);
-#endif
 #ifdef DEBUG_EXTENSIONS
   scm_last_debug_frame = SCM_DFRAME (old_rootcont);
 #endif
@@ -335,7 +319,7 @@ cwdr (SCM proc, SCM a1, SCM args, SCM handler, SCM_STACKITEM *stack_start)
 
 SCM_DEFINE (scm_call_with_dynamic_root, "call-with-dynamic-root", 2, 0, 0,
            (SCM thunk, SCM handler),
-	    "Evaluate @var{(thunk)} in a new dynamic context, returning its value.\n\n"
+	    "Evaluate @code{(thunk)} in a new dynamic context, returning its value.\n\n"
 	    "If an error occurs during evaluation, apply @var{handler} to the\n"
 	    "arguments to the throw, just as @code{throw} would.  If this happens,\n"
 	    "@var{handler} is called outside the scope of the new root -- it is\n"
@@ -348,7 +332,7 @@ SCM_DEFINE (scm_call_with_dynamic_root, "call-with-dynamic-root", 2, 0, 0,
 	    "Before calling @var{thunk}, the dynamic-wind chain is un-wound back to\n"
 	    "the root and a new chain started for @var{thunk}.  Therefore, this call\n"
 	    "may not do what you expect:\n\n"
-	    "@example\n"
+	    "@lisp\n"
 	    ";; Almost certainly a bug:\n"
 	    "(with-output-to-port\n"
 	    " some-port\n\n"
@@ -358,9 +342,8 @@ SCM_DEFINE (scm_call_with_dynamic_root, "call-with-dynamic-root", 2, 0, 0,
 	    "      (display 'fnord)\n"
 	    "      (newline))\n"
 	    "    (lambda (errcode) errcode))))\n"
-	    "@end example\n\n"
-	    "The problem is, on what port will @samp{fnord\n"
-	    "} be displayed?  You\n"
+	    "@end lisp\n\n"
+	    "The problem is, on what port will @samp{fnord} be displayed?  You\n"
 	    "might expect that because of the @code{with-output-to-port} that\n"
 	    "it will be displayed on the port bound to @code{some-port}.  But it\n"
 	    "probably won't -- before evaluating the thunk, dynamic winds are\n"
@@ -399,12 +382,13 @@ scm_apply_with_dynamic_root (SCM proc, SCM a1, SCM args, SCM handler)
 
 
 
+#if (SCM_DEBUG_DEPRECATED == 0)
+
 /* Call thunk(closure) underneath a top-level error handler.
  * If an error occurs, pass the exitval through err_filter and return it.
  * If no error occurs, return the value of thunk.
  */
 
-
 #ifdef _UNICOS
 typedef int setjmp_type;
 #else
@@ -412,7 +396,6 @@ typedef long setjmp_type;
 #endif
 
 
-
 SCM
 scm_call_catching_errors (SCM (*thunk)(), SCM (*err_filter)(), void *closure)
 {
@@ -436,12 +419,16 @@ scm_call_catching_errors (SCM (*thunk)(), SCM (*err_filter)(), void *closure)
   return answer;
 }
 
+#endif  /* SCM_DEBUG_DEPRECATED == 0 */
+
+
 void
 scm_init_root ()
 {
-  scm_tc16_root = scm_make_smob_type_mfpe ("root", sizeof (struct scm_root_state),
-                                          mark_root, NULL, print_root, NULL);
-                                          
+  scm_tc16_root = scm_make_smob_type ("root", sizeof (struct scm_root_state));
+  scm_set_smob_mark (scm_tc16_root, root_mark);
+  scm_set_smob_print (scm_tc16_root, root_print);
+
 #include "libguile/root.x"
 }
 
