@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995,1996,1997,1998, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1995,1996,1997,1998,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,7 +42,8 @@
 
 
 
-#include <stdio.h>
+#include <errno.h>
+
 #include "libguile/_scm.h"
 
 #include "libguile/scmsigs.h"
@@ -65,11 +66,13 @@ extern int system();
 #ifdef HAVE_SYSTEM
 SCM_DEFINE (scm_system, "system", 0, 1, 0, 
            (SCM cmd),
-	    "Executes @var{cmd} using the operating system's \"command processor\".\n"
-	    "Under Unix this is usually the default shell @code{sh}.  The value\n"
-	    "returned is @var{cmd}'s exit status as returned by @code{waitpid}, which\n"
-	    "can be interpreted using the functions above.\n\n"
-	    "If @code{system} is called without arguments, it returns a boolean\n"
+	    "Execute @var{cmd} using the operating system's \"command\n"
+	    "processor\".  Under Unix this is usually the default shell\n"
+	    "@code{sh}.  The value returned is @var{cmd}'s exit status as\n"
+	    "returned by @code{waitpid}, which can be interpreted using the\n"
+	    "functions above.\n"
+	    "\n"
+	    "If @code{system} is called without arguments, return a boolean\n"
 	    "indicating whether the command processor is available.")
 #define FUNC_NAME s_scm_system
 {
@@ -80,12 +83,11 @@ SCM_DEFINE (scm_system, "system", 0, 1, 0,
       rv = system (NULL);
       return SCM_BOOL(rv);
     }
-  SCM_VALIDATE_ROSTRING (1,cmd);
+  SCM_VALIDATE_STRING (1, cmd);
   SCM_DEFER_INTS;
   errno = 0;
-  if (SCM_ROSTRINGP (cmd))
-    cmd = scm_makfromstr (SCM_ROCHARS (cmd), SCM_ROLENGTH (cmd), 0);
-  rv = system(SCM_ROCHARS(cmd));
+  SCM_STRING_COERCE_0TERMINATION_X (cmd);
+  rv = system (SCM_STRING_CHARS (cmd));
   if (rv == -1 || (rv == 127 && errno != 0))
     SCM_SYSERROR;
   SCM_ALLOW_INTS;
@@ -99,15 +101,14 @@ SCM_DEFINE (scm_getenv, "getenv", 1, 0, 0,
             (SCM nam),
 	    "Looks up the string @var{name} in the current environment.  The return\n"
 	    "value is @code{#f} unless a string of the form @code{NAME=VALUE} is\n"
-	    "found, in which case the string @code{VALUE} is\n"
-	    "returned.")
+	    "found, in which case the string @code{VALUE} is returned.")
 #define FUNC_NAME s_scm_getenv
 {
   char *val;
-  SCM_VALIDATE_ROSTRING (1,nam);
-  nam = scm_makfromstr (SCM_ROCHARS (nam), SCM_ROLENGTH (nam), 0);
-  val = getenv(SCM_CHARS(nam));
-  return (val) ? scm_makfromstr(val, (scm_sizet)strlen(val), 0) : SCM_BOOL_F;
+  SCM_VALIDATE_STRING (1, nam);
+  SCM_STRING_COERCE_0TERMINATION_X (nam);
+  val = getenv (SCM_STRING_CHARS (nam));
+  return val ? scm_mem2string (val, strlen (val)) : SCM_BOOL_F;
 }
 #undef FUNC_NAME
 
