@@ -1,4 +1,4 @@
-/*	Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2002 Free Software Foundation
+/* Copyright (C) 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2005 Free Software Foundation
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -45,7 +45,8 @@
 
 
 
-#include <stdio.h>
+#include <errno.h>
+
 #include "libguile/_scm.h"
 #include "libguile/smob.h"
 #include "libguile/alist.h"
@@ -74,19 +75,19 @@
  *
  */
 
-SCM scm_sym_filename;
-SCM scm_sym_copy;
-SCM scm_sym_line;
-SCM scm_sym_column;
-SCM scm_sym_breakpoint;
+SCM_GLOBAL_SYMBOL (scm_sym_filename, "filename");
+SCM_GLOBAL_SYMBOL (scm_sym_copy, "copy");
+SCM_GLOBAL_SYMBOL (scm_sym_line, "line");
+SCM_GLOBAL_SYMBOL (scm_sym_column, "column");
+SCM_GLOBAL_SYMBOL (scm_sym_breakpoint, "breakpoint");
 
-long scm_tc16_srcprops;
-static scm_srcprops_chunk *srcprops_chunklist = 0;
-static scm_srcprops *srcprops_freelist = 0;
+scm_t_bits scm_tc16_srcprops;
+static scm_t_srcprops_chunk *srcprops_chunklist = 0;
+static scm_t_srcprops *srcprops_freelist = 0;
 
 
 static SCM
-marksrcprops (SCM obj)
+srcprops_mark (SCM obj)
 {
   scm_gc_mark (SRCPROPFNAME (obj));
   scm_gc_mark (SRCPROPCOPY (obj));
@@ -94,17 +95,17 @@ marksrcprops (SCM obj)
 }
 
 
-static scm_sizet
-freesrcprops (SCM obj)
+static size_t
+srcprops_free (SCM obj)
 {
-  *((scm_srcprops **) SCM_CELL_WORD_1 (obj)) = srcprops_freelist;
-  srcprops_freelist = (scm_srcprops *) SCM_CELL_WORD_1 (obj);
+  *((scm_t_srcprops **) SCM_CELL_WORD_1 (obj)) = srcprops_freelist;
+  srcprops_freelist = (scm_t_srcprops *) SCM_CELL_WORD_1 (obj);
   return 0; /* srcprops_chunks are not freed until leaving guile */
 }
 
 
 static int
-prinsrcprops (SCM obj,SCM port,scm_print_state *pstate)
+srcprops_print (SCM obj, SCM port, scm_print_state *pstate)
 {
   int writingp = SCM_WRITINGP (pstate);
   scm_puts ("#<srcprops ", port);
@@ -117,33 +118,35 @@ prinsrcprops (SCM obj,SCM port,scm_print_state *pstate)
 
 
 SCM
-scm_make_srcprops (int line, int col, SCM filename, SCM copy, SCM plist)
+scm_make_srcprops (long line, int col, SCM filename, SCM copy, SCM plist)
 {
-  register scm_srcprops *ptr;
+  register scm_t_srcprops *ptr;
   SCM_DEFER_INTS;
   if ((ptr = srcprops_freelist) != NULL)
-    srcprops_freelist = *(scm_srcprops **)ptr;
+    srcprops_freelist = *(scm_t_srcprops **)ptr;
   else
     {
-      int i;
-      scm_srcprops_chunk *mem;
-      scm_sizet n = sizeof (scm_srcprops_chunk)
-	            + sizeof (scm_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
-      SCM_SYSCALL (mem = (scm_srcprops_chunk *) malloc (n));
-      SCM_ASSERT (mem, SCM_UNDEFINED, SCM_NALLOC, "srcprops");
+      size_t i;
+      scm_t_srcprops_chunk *mem;
+      size_t n = sizeof (scm_t_srcprops_chunk)
+	            + sizeof (scm_t_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
+      SCM_SYSCALL (mem = (scm_t_srcprops_chunk *) malloc (n));
+      if (mem == NULL)
+	scm_memory_error ("srcprops");
       scm_mallocated += n;
       mem->next = srcprops_chunklist;
       srcprops_chunklist = mem;
       ptr = &mem->srcprops[0];
       for (i = 1; i < SRCPROPS_CHUNKSIZE - 1; ++i)
-	*(scm_srcprops **)&ptr[i] = &ptr[i + 1];
-      *(scm_srcprops **)&ptr[SRCPROPS_CHUNKSIZE - 1] = 0;
-      srcprops_freelist = (scm_srcprops *) &ptr[1];
+	*(scm_t_srcprops **)&ptr[i] = &ptr[i + 1];
+      *(scm_t_srcprops **)&ptr[SRCPROPS_CHUNKSIZE - 1] = 0;
+      srcprops_freelist = (scm_t_srcprops *) &ptr[1];
     }
   ptr->pos = SRCPROPMAKPOS (line, col);
   ptr->fname = filename;
   ptr->copy = copy;
   ptr->plist = plist;
+  SCM_ALLOW_INTS;
   SCM_RETURN_NEWSMOB (scm_tc16_srcprops, ptr);
 }
 
@@ -164,7 +167,7 @@ scm_srcprops_to_plist (SCM obj)
 
 SCM_DEFINE (scm_source_properties, "source-properties", 1, 0, 0, 
             (SCM obj),
-"")
+	    "Return the source property association list of @var{obj}.")
 #define FUNC_NAME s_scm_source_properties
 {
   SCM p;
@@ -172,13 +175,15 @@ SCM_DEFINE (scm_source_properties, "source-properties", 1, 0, 0,
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
 #ifndef SCM_RECKLESS
-  else if (SCM_NCONSP (obj))
+  else if (SCM_NECONSP (obj))
     SCM_WRONG_TYPE_ARG (1, obj);
 #endif
-  p = scm_hashq_ref (scm_source_whash, obj, SCM_BOOL_F);
+  p = scm_hashq_ref (scm_source_whash, obj, SCM_EOL);
   if (SRCPROPSP (p))
     return scm_srcprops_to_plist (p);
-  return SCM_EOL;
+  else
+    /* list from set-source-properties!, or SCM_EOL for not found */
+    return p;
 }
 #undef FUNC_NAME
 
@@ -186,7 +191,8 @@ SCM_DEFINE (scm_source_properties, "source-properties", 1, 0, 0,
    and try to make a srcprops-object...? */
 SCM_DEFINE (scm_set_source_properties_x, "set-source-properties!", 2, 0, 0,
             (SCM obj, SCM plist),
-"")
+	    "Install the association list @var{plist} as the source property\n"
+	    "list for @var{obj}.")
 #define FUNC_NAME s_scm_set_source_properties_x
 {
   SCM handle;
@@ -194,7 +200,7 @@ SCM_DEFINE (scm_set_source_properties_x, "set-source-properties!", 2, 0, 0,
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
 #ifndef SCM_RECKLESS
-  else if (SCM_NCONSP (obj))
+  else if (SCM_NECONSP (obj))
     SCM_WRONG_TYPE_ARG(1, obj);
 #endif
   handle = scm_hashq_create_handle_x (scm_source_whash, obj, plist);
@@ -205,7 +211,8 @@ SCM_DEFINE (scm_set_source_properties_x, "set-source-properties!", 2, 0, 0,
 
 SCM_DEFINE (scm_source_property, "source-property", 2, 0, 0,
             (SCM obj, SCM key),
-"")
+	    "Return the source property specified by @var{key} from\n"
+	    "@var{obj}'s source property list.")
 #define FUNC_NAME s_scm_source_property
 {
   SCM p;
@@ -237,7 +244,8 @@ SCM_DEFINE (scm_source_property, "source-property", 2, 0, 0,
 
 SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
             (SCM obj, SCM key, SCM datum),
-"")
+	    "Set the source property of object @var{obj}, which is specified by\n"
+	    "@var{key} to @var{datum}.  Normally, the key will be a symbol.")
 #define FUNC_NAME s_scm_set_source_property_x
 {
   scm_whash_handle h;
@@ -246,7 +254,7 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
   if (SCM_MEMOIZEDP (obj))
     obj = SCM_MEMOIZED_EXP (obj);
 #ifndef SCM_RECKLESS
-  else if (SCM_NCONSP (obj))
+  else if (SCM_NECONSP (obj))
     SCM_WRONG_TYPE_ARG (1, obj);
 #endif
   h = scm_whash_get_handle (scm_source_whash, obj);
@@ -259,24 +267,22 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
     }
   if (SCM_EQ_P (scm_sym_breakpoint, key))
     {
-      if (SCM_FALSEP (datum))
-	CLEARSRCPROPBRK (SRCPROPSP (p)
-			 ? p
-			 : SCM_WHASHSET (scm_source_whash, h,
-					 scm_make_srcprops (0,
-							    0,
-							    SCM_UNDEFINED,
-							    SCM_UNDEFINED,
-							    p)));
+      if (SRCPROPSP (p))
+	{
+	  if (SCM_FALSEP (datum))
+	    CLEARSRCPROPBRK (p);
+	  else
+	    SETSRCPROPBRK (p);
+	}
       else
-	SETSRCPROPBRK (SRCPROPSP (p)
-		       ? p
-		       : SCM_WHASHSET (scm_source_whash, h,
-				       scm_make_srcprops (0,
-							  0,
-							  SCM_UNDEFINED,
-							  SCM_UNDEFINED,
-							  p)));
+	{
+	  SCM sp = scm_make_srcprops (0, 0, SCM_UNDEFINED, SCM_UNDEFINED, p);
+	  SCM_WHASHSET (scm_source_whash, h, sp);
+	  if (SCM_FALSEP (datum))
+	    CLEARSRCPROPBRK (sp);
+	  else
+	    SETSRCPROPBRK (sp);
+	}
     }
   else if (SCM_EQ_P (scm_sym_line, key))
     {
@@ -305,7 +311,7 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
       else
 	SCM_WHASHSET (scm_source_whash, h, scm_make_srcprops (0, 0, datum, SCM_UNDEFINED, p));
     }
-  else if (SCM_EQ_P (scm_sym_filename, key))
+  else if (SCM_EQ_P (scm_sym_copy, key))
     {
       if (SRCPROPSP (p))
 	SRCPROPCOPY (p) = datum;
@@ -322,30 +328,27 @@ SCM_DEFINE (scm_set_source_property_x, "set-source-property!", 3, 0, 0,
 void
 scm_init_srcprop ()
 {
-  scm_tc16_srcprops = scm_make_smob_type_mfpe ("srcprops", 0,
-                                         marksrcprops, freesrcprops, prinsrcprops, NULL);
+  scm_tc16_srcprops = scm_make_smob_type ("srcprops", 0);
+  scm_set_smob_mark (scm_tc16_srcprops, srcprops_mark);
+  scm_set_smob_free (scm_tc16_srcprops, srcprops_free);
+  scm_set_smob_print (scm_tc16_srcprops, srcprops_print);
+
   scm_source_whash = scm_make_weak_key_hash_table (SCM_MAKINUM (2047));
+  scm_c_define ("source-whash", scm_source_whash);
 
-  scm_sym_filename = SCM_CAR (scm_sysintern ("filename", SCM_UNDEFINED));
-  scm_sym_copy = SCM_CAR (scm_sysintern ("copy", SCM_UNDEFINED));
-  scm_sym_line = SCM_CAR (scm_sysintern ("line", SCM_UNDEFINED));
-  scm_sym_column = SCM_CAR (scm_sysintern ("column", SCM_UNDEFINED));
-  scm_sym_breakpoint = SCM_CAR (scm_sysintern ("breakpoint", SCM_UNDEFINED));
-
-  scm_sysintern ("source-whash", scm_source_whash);
 #include "libguile/srcprop.x"
 }
 
 void
 scm_finish_srcprop ()
 {
-  register scm_srcprops_chunk *ptr = srcprops_chunklist, *next;
+  register scm_t_srcprops_chunk *ptr = srcprops_chunklist, *next;
   while (ptr)
     {
       next = ptr->next;
       free ((char *) ptr);
-      scm_mallocated -= sizeof (scm_srcprops_chunk)
-	                + sizeof (scm_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
+      scm_mallocated -= sizeof (scm_t_srcprops_chunk)
+	                + sizeof (scm_t_srcprops) * (SRCPROPS_CHUNKSIZE - 1);
       ptr = next;
     }
 }
