@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef SRCPROPH
-#define SRCPROPH
-/*	Copyright (C) 1995,1996, 2000, 2002 Free Software Foundation
+#ifndef SCM_SOURCE_PROPERTIES_H
+#define SCM_SOURCE_PROPERTIES_H
+/* Copyright (C) 1995,1996,2000,2001 Free Software Foundation
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -76,33 +76,45 @@ do { \
 /* {Source properties}
  */
 
-extern long scm_tc16_srcprops;
+extern scm_t_bits scm_tc16_srcprops;
 
-typedef struct scm_srcprops
+typedef struct scm_t_srcprops
 {
   unsigned long pos;
   SCM fname;
   SCM copy;
   SCM plist;
-} scm_srcprops;
+} scm_t_srcprops;
 
 #define SRCPROPS_CHUNKSIZE 2047 /* Number of srcprops per chunk */
-typedef struct scm_srcprops_chunk
+typedef struct scm_t_srcprops_chunk
 {
-  struct scm_srcprops_chunk *next;
-  scm_srcprops srcprops[1];
-} scm_srcprops_chunk;
+  struct scm_t_srcprops_chunk *next;
+  scm_t_srcprops srcprops[1];
+} scm_t_srcprops_chunk;
 
-#define SRCPROPSP(p) (SCM_NIMP(p) && (SCM_TYP16 (p) == scm_tc16_srcprops))
-#define SRCPROPBRK(p) (SCM_BOOL (SCM_CELL_WORD_0 (p) & (1L << 16)))
-#define SRCPROPPOS(p) ((scm_srcprops *) SCM_CELL_WORD_1 (p))->pos
+#if (SCM_DEBUG_DEPRECATED == 0)
+# define scm_srcprops scm_t_srcprops
+# define scm_srcprops_chunk scm_t_srcprops_chunk
+#endif
+
+#define SCM_SOURCE_PROPERTY_FLAG_BREAK (1L << 16)
+
+#define SRCPROPSP(p) (SCM_TYP16_PREDICATE (scm_tc16_srcprops, p))
+#define SRCPROPBRK(p) \
+  (SCM_BOOL (SCM_CELL_WORD_0 (p) & SCM_SOURCE_PROPERTY_FLAG_BREAK))
+#define SRCPROPPOS(p) ((scm_t_srcprops *) SCM_CELL_WORD_1 (p))->pos
 #define SRCPROPLINE(p) (SRCPROPPOS(p) >> 12)
 #define SRCPROPCOL(p) (SRCPROPPOS(p) & 0x0fffL)
-#define SRCPROPFNAME(p) ((scm_srcprops *) SCM_CELL_WORD_1 (p))->fname
-#define SRCPROPCOPY(p) ((scm_srcprops *) SCM_CELL_WORD_1 (p))->copy
-#define SRCPROPPLIST(p) ((scm_srcprops *) SCM_CELL_WORD_1 (p))->plist
-#define SETSRCPROPBRK(p) (SCM_SETOR_CAR (p, (1L << 16)))
-#define CLEARSRCPROPBRK(p) SCM_SETAND_CAR (p, ~(1L << 16))
+#define SRCPROPFNAME(p) ((scm_t_srcprops *) SCM_CELL_WORD_1 (p))->fname
+#define SRCPROPCOPY(p) ((scm_t_srcprops *) SCM_CELL_WORD_1 (p))->copy
+#define SRCPROPPLIST(p) ((scm_t_srcprops *) SCM_CELL_WORD_1 (p))->plist
+#define SETSRCPROPBRK(p) \
+  (SCM_SET_CELL_WORD_0 ((p), SCM_CELL_WORD_0 (p) \
+                             | SCM_SOURCE_PROPERTY_FLAG_BREAK))
+#define CLEARSRCPROPBRK(p)  \
+  (SCM_SET_CELL_WORD_0 ((p), SCM_CELL_WORD_0 (p) \
+                             & ~SCM_SOURCE_PROPERTY_FLAG_BREAK))
 #define SRCPROPMAKPOS(l,c) (((l) << 12) + (c))
 #define SETSRCPROPPOS(p,l,c) (SRCPROPPOS (p) = SRCPROPMAKPOS (l, c))
 #define SETSRCPROPLINE(p,l) SETSRCPROPPOS (p, l, SRCPROPCOL (p))
@@ -124,7 +136,7 @@ extern SCM scm_sym_breakpoint;
 
 
 extern SCM scm_srcprops_to_plist (SCM obj);
-extern SCM scm_make_srcprops (int line, int col, SCM fname, SCM copy, SCM plist);
+extern SCM scm_make_srcprops (long line, int col, SCM fname, SCM copy, SCM plist);
 extern SCM scm_source_property (SCM obj, SCM key);
 extern SCM scm_set_source_property_x (SCM obj, SCM key, SCM datum);
 extern SCM scm_source_properties (SCM obj);
@@ -132,7 +144,7 @@ extern SCM scm_set_source_properties_x (SCM obj, SCM props);
 extern void scm_finish_srcprop (void);
 extern void scm_init_srcprop (void);
 
-#endif /* SRCPROPH */
+#endif /* SCM_SOURCE_PROPERTIES_H */
 
 /*
   Local Variables:
