@@ -1,4 +1,4 @@
-/* Copyright (C) 1996, 97, 98, 99, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1996,1997,1998,1999,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -12,8 +12,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -42,7 +42,6 @@
 
 
 
-#include <stdio.h>
 #include "libguile/_scm.h"
 #include "libguile/chars.h"
 #include "libguile/eval.h"
@@ -68,26 +67,30 @@ SCM scm_struct_table;
 SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0, 
             (SCM fields),
 	    "Return a new structure layout object.\n\n"
-	    "@var{fields} must be a read-only string made up of pairs of characters\n"
+	    "@var{fields} must be a string made up of pairs of characters\n"
 	    "strung together.  The first character of each pair describes a field\n"
 	    "type, the second a field protection.  Allowed types are 'p' for\n"
 	    "GC-protected Scheme data, 'u' for unprotected binary data, and 's' for\n"
-	    "fields that should point to the structure itself.    Allowed protections\n"
-	    "are 'w' for mutable fields, 'r' for read-only fields, and 'o' for opaque \n"
+	    "a field that points to the structure itself.    Allowed protections\n"
+	    "are 'w' for mutable fields, 'r' for read-only fields, and 'o' for opaque\n"
 	    "fields.  The last field protection specification may be capitalized to\n"
 	    "indicate that the field is a tail-array.")
 #define FUNC_NAME s_scm_make_struct_layout
 {
   SCM new_sym;
-  SCM_VALIDATE_ROSTRING (1,fields);
+  SCM_VALIDATE_STRING (1, fields);
+
   { /* scope */
     char * field_desc;
-    int len;
+    size_t len;
     int x;
 
-    len = SCM_ROLENGTH (fields);
-    field_desc = SCM_ROCHARS (fields);
-    SCM_ASSERT (!(len & 1), fields, "odd length field specification", FUNC_NAME);
+    len = SCM_STRING_LENGTH (fields);
+    if (len % 2 == 1)
+      SCM_MISC_ERROR ("odd length field specification: ~S", 
+		      scm_list_1 (fields));
+
+    field_desc = SCM_STRING_CHARS (fields);
 
     for (x = 0; x < len; x += 2)
       {
@@ -102,41 +105,44 @@ SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0,
 	  case 's':
 	    break;
 	  default:
-	    SCM_ASSERT (0, SCM_MAKE_CHAR (field_desc[x]) , "unrecognized field type", FUNC_NAME);
+	    SCM_MISC_ERROR ("unrecognized field type: ~S", 
+			    scm_list_1 (SCM_MAKE_CHAR (field_desc[x])));
 	  }
 
 	switch (field_desc[x + 1])
 	  {
 	  case 'w':
-	    SCM_ASSERT (field_desc[x] != 's', SCM_MAKE_CHAR (field_desc[x + 1]),
-			"self fields not writable", FUNC_NAME);
-	      
+	    if (field_desc[x] == 's')
+	      SCM_MISC_ERROR ("self fields not writable", SCM_EOL);
 	  case 'r':
 	  case 'o':
 	    break;
 	  case 'R':
 	  case 'W':
 	  case 'O':
-	    SCM_ASSERT (field_desc[x] != 's', SCM_MAKE_CHAR (field_desc[x + 1]),
-			"self fields not allowed in tail array",
-                        FUNC_NAME);
-	    SCM_ASSERT (x == len - 2, SCM_MAKE_CHAR (field_desc[x + 1]),
-			"tail array field must be last field in layout",
-                        FUNC_NAME);
+	    if (field_desc[x] == 's')
+	      SCM_MISC_ERROR ("self fields not allowed in tail array", 
+			      SCM_EOL);
+	    if (x != len - 2)
+	      SCM_MISC_ERROR ("tail array field must be last field in layout",
+			      SCM_EOL);
 	    break;
 	  default:
-	    SCM_ASSERT (0, SCM_MAKE_CHAR (field_desc[x]) , "unrecognized ref specification", FUNC_NAME);
+	    SCM_MISC_ERROR ("unrecognized ref specification: ~S",
+			    scm_list_1 (SCM_MAKE_CHAR (field_desc[x + 1])));
 	  }
 #if 0
 	if (field_desc[x] == 'd')
 	  {
-	    SCM_ASSERT (field_desc[x + 2] == '-', SCM_MAKINUM (x / 2), "missing dash field", FUNC_NAME);
+	    if (field_desc[x + 2] != '-')
+	      SCM_MISC_ERROR ("missing dash field at position ~A",
+			      scm_list_1 (SCM_MAKINUM (x / 2)));
 	    x += 2;
 	    goto recheck_ref;
 	  }
 #endif
       }
-    new_sym = SCM_CAR (scm_intern_obarray (field_desc, len, SCM_BOOL_F));
+    new_sym = scm_mem2symbol (field_desc, len);
   }
   return scm_return_first (new_sym, fields);
 }
@@ -146,14 +152,12 @@ SCM_DEFINE (scm_make_struct_layout, "make-struct-layout", 1, 0, 0,
 
 
 
-void
-scm_struct_init (SCM handle, int tail_elts, SCM inits)
+static void
+scm_struct_init (SCM handle, SCM layout, scm_t_bits * mem, int tail_elts, SCM inits)
 {
-  SCM layout = SCM_STRUCT_LAYOUT (handle);
-  unsigned char * fields_desc = (unsigned char *) SCM_CHARS (layout) - 2;
+  unsigned char * fields_desc = (unsigned char *) SCM_SYMBOL_CHARS (layout) - 2;
   unsigned char prot = 0;
-  int n_fields = SCM_LENGTH (layout) / 2;
-  scm_bits_t * mem = SCM_STRUCT_DATA (handle);
+  int n_fields = SCM_SYMBOL_LENGTH (layout) / 2;
   int tailp = 0;
 
   while (n_fields)
@@ -236,7 +240,8 @@ scm_struct_init (SCM handle, int tail_elts, SCM inits)
 
 SCM_DEFINE (scm_struct_p, "struct?", 1, 0, 0, 
             (SCM x),
-	    "Return #t iff @var{obj} is a structure object, else #f.")
+	    "Return @code{#t} iff @var{x} is a structure object, else\n"
+	    "@code{#f}.")
 #define FUNC_NAME s_scm_struct_p
 {
   return SCM_BOOL(SCM_STRUCTP (x));
@@ -245,22 +250,22 @@ SCM_DEFINE (scm_struct_p, "struct?", 1, 0, 0,
 
 SCM_DEFINE (scm_struct_vtable_p, "struct-vtable?", 1, 0, 0, 
             (SCM x),
-	    "Return #t iff obj is a vtable structure.")
+	    "Return @code{#t} iff @var{x} is a vtable structure.")
 #define FUNC_NAME s_scm_struct_vtable_p
 {
   SCM layout;
-  scm_bits_t * mem;
+  scm_t_bits * mem;
 
   if (!SCM_STRUCTP (x))
     return SCM_BOOL_F;
 
   layout = SCM_STRUCT_LAYOUT (x);
 
-  if (SCM_LENGTH (layout) < SCM_LENGTH (required_vtable_fields))
+  if (SCM_SYMBOL_LENGTH (layout) < SCM_STRING_LENGTH (required_vtable_fields))
     return SCM_BOOL_F;
 
-  if (strncmp (SCM_CHARS (layout), SCM_CHARS (required_vtable_fields),
-	       SCM_LENGTH (required_vtable_fields)))
+  if (strncmp (SCM_SYMBOL_CHARS (layout), SCM_STRING_CHARS (required_vtable_fields),
+	       SCM_STRING_LENGTH (required_vtable_fields)))
     return SCM_BOOL_F;
 
   mem = SCM_STRUCT_DATA (x);
@@ -303,76 +308,142 @@ SCM_DEFINE (scm_struct_vtable_p, "struct-vtable?", 1, 0, 0,
      Ugh.  */
 
 
-scm_bits_t *
+scm_t_bits *
 scm_alloc_struct (int n_words, int n_extra, char *who)
 {
-  int size = sizeof (scm_bits_t) * (n_words + n_extra) + 7;
+  int size = sizeof (scm_t_bits) * (n_words + n_extra) + 7;
   void * block = scm_must_malloc (size, who);
 
   /* Adjust the pointer to hide the extra words.  */
-  scm_bits_t * p = (scm_bits_t *) block + n_extra;
+  scm_t_bits * p = (scm_t_bits *) block + n_extra;
 
   /* Adjust it even further so it's aligned on an eight-byte boundary.  */
-  p = (scm_bits_t *) (((scm_bits_t) p + 7) & ~7);
+  p = (scm_t_bits *) (((scm_t_bits) p + 7) & ~7);
 
   /* Initialize a few fields as described above.  */
-  p[scm_struct_i_free] = (scm_bits_t) scm_struct_free_standard;
-  p[scm_struct_i_ptr] = (scm_bits_t) block;
+  p[scm_struct_i_free] = (scm_t_bits) scm_struct_free_standard;
+  p[scm_struct_i_ptr] = (scm_t_bits) block;
   p[scm_struct_i_n_words] = n_words;
   p[scm_struct_i_flags] = 0;
 
   return p;
 }
 
-scm_sizet
-scm_struct_free_0 (scm_bits_t * vtable, scm_bits_t * data)
+size_t
+scm_struct_free_0 (scm_t_bits * vtable SCM_UNUSED,
+		   scm_t_bits * data SCM_UNUSED)
 {
   return 0;
 }
 
-scm_sizet
-scm_struct_free_light (scm_bits_t * vtable, scm_bits_t * data)
+size_t
+scm_struct_free_light (scm_t_bits * vtable, scm_t_bits * data)
 {
   scm_must_free (data);
   return vtable [scm_struct_i_size] & ~SCM_STRUCTF_MASK;
 }
 
-scm_sizet
-scm_struct_free_standard (scm_bits_t * vtable, scm_bits_t * data)
+size_t
+scm_struct_free_standard (scm_t_bits * vtable SCM_UNUSED, scm_t_bits * data)
 {
   size_t n = (data[scm_struct_i_n_words] + scm_struct_n_extra_words)
-	     * sizeof (scm_bits_t) + 7;
+	     * sizeof (scm_t_bits) + 7;
   scm_must_free ((void *) data[scm_struct_i_ptr]);
   return n;
 }
 
-scm_sizet
-scm_struct_free_entity (scm_bits_t * vtable, scm_bits_t * data)
+size_t
+scm_struct_free_entity (scm_t_bits * vtable SCM_UNUSED, scm_t_bits * data)
 {
   size_t n = (data[scm_struct_i_n_words] + scm_struct_entity_n_extra_words)
-	     * sizeof (scm_bits_t) + 7;
+	     * sizeof (scm_t_bits) + 7;
   scm_must_free ((void *) data[scm_struct_i_ptr]);
   return n;
 }
 
+static void *
+scm_struct_gc_init (void *dummy1 SCM_UNUSED,
+		    void *dummy2 SCM_UNUSED,
+		    void *dummy3 SCM_UNUSED)
+{
+  scm_structs_to_free = SCM_EOL;
+  return 0;
+}
+
+static void *
+scm_free_structs (void *dummy1 SCM_UNUSED,
+		  void *dummy2 SCM_UNUSED,
+		  void *dummy3 SCM_UNUSED)
+{
+  SCM newchain = scm_structs_to_free;
+  do
+    {
+      /* Mark vtables in GC chain.  GC mark set means delay freeing. */
+      SCM chain = newchain;
+      while (!SCM_NULLP (chain))
+	{
+	  SCM vtable = SCM_STRUCT_VTABLE (chain);
+	  if (SCM_STRUCT_GC_CHAIN (vtable) != 0 && vtable != chain)
+	    SCM_SETGCMARK (vtable);
+	  chain = SCM_STRUCT_GC_CHAIN (chain);
+	}
+      /* Free unmarked structs.  */
+      chain = newchain;
+      newchain = SCM_EOL;
+      while (!SCM_NULLP (chain))
+	{
+	  SCM obj = chain;
+	  chain = SCM_STRUCT_GC_CHAIN (chain);
+	  if (SCM_GCMARKP (obj))
+	    {
+	      SCM_CLRGCMARK (obj);
+	      SCM_SET_STRUCT_GC_CHAIN (obj, newchain);
+	      newchain = obj;
+	    }
+	  else
+	    {
+	      scm_t_bits word0 = SCM_CELL_WORD_0 (obj) - scm_tc3_cons_gloc;
+	      /* access as struct */
+	      scm_t_bits * vtable_data = (scm_t_bits *) word0;
+	      scm_t_bits * data = SCM_STRUCT_DATA (obj);
+	      scm_t_struct_free free_struct_data
+		= ((scm_t_struct_free) vtable_data[scm_struct_i_free]);
+	      SCM_SET_CELL_TYPE (obj, scm_tc_free_cell);
+	      free_struct_data (vtable_data, data);
+	    }
+	}
+    }
+  while (!SCM_NULLP (newchain));
+  return 0;
+}
+
 SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1, 
             (SCM vtable, SCM tail_array_size, SCM init),
 	    "Create a new structure.\n\n"
-	    "@var{type} must be a vtable structure (@xref{Vtables}).\n\n"
+	    "@var{type} must be a vtable structure (@pxref{Vtables}).\n\n"
 	    "@var{tail-elts} must be a non-negative integer.  If the layout\n"
 	    "specification indicated by @var{type} includes a tail-array,\n"
 	    "this is the number of elements allocated to that array.\n\n"
-	    "The @var{inits} are optional arguments describing how successive fields\n"
-	    "of the structure should be initialized.  Only fields with protection 'r'\n"
-	    "or 'w' can be initialized -- fields of protection 's' are automatically\n"
-	    "initialized to point to the new structure itself;  fields of protection 'o'\n"
-	    "can not be initialized by Scheme programs.")
+	    "The @var{init1}, @dots{} are optional arguments describing how\n"
+	    "successive fields of the structure should be initialized.  Only fields\n"
+	    "with protection 'r' or 'w' can be initialized, except for fields of\n"
+	    "type 's', which are automatically initialized to point to the new\n"
+	    "structure itself; fields with protection 'o' can not be initialized by\n"
+	    "Scheme programs.\n\n"
+	    "If fewer optional arguments than initializable fields are supplied,\n"
+	    "fields of type 'p' get default value #f while fields of type 'u' are\n"
+	    "initialized to 0.\n\n"
+	    "Structs are currently the basic representation for record-like data\n"
+	    "structures in Guile.  The plan is to eventually replace them with a\n"
+	    "new representation which will at the same time be easier to use and\n"
+	    "more powerful.\n\n"
+	    "For more information, see the documentation for @code{make-vtable-vtable}.")
 #define FUNC_NAME s_scm_make_struct
 {
   SCM layout;
   int basic_size;
   int tail_elts;
-  scm_bits_t * data;
+  scm_t_bits * data;
   SCM handle;
 
   SCM_VALIDATE_VTABLE (1,vtable);
@@ -380,9 +451,9 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
   SCM_VALIDATE_REST_ARGUMENT (init);
 
   layout = SCM_PACK (SCM_STRUCT_DATA (vtable) [scm_vtable_index_layout]);
-  basic_size = SCM_LENGTH (layout) / 2;
+  basic_size = SCM_SYMBOL_LENGTH (layout) / 2;
   tail_elts = SCM_INUM (tail_array_size);
-  SCM_NEWCELL (handle);
+  SCM_NEWCELL2 (handle);
   SCM_DEFER_INTS;
   if (SCM_STRUCT_DATA (vtable)[scm_struct_i_flags] & SCM_STRUCTF_ENTITY)
     {
@@ -397,8 +468,9 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
 			     scm_struct_n_extra_words,
 			     "make-struct");
   SCM_SET_CELL_WORD_1 (handle, data);
-  SCM_SET_CELL_WORD_0 (handle, (scm_bits_t) SCM_STRUCT_DATA (vtable) + scm_tc3_cons_gloc);
-  scm_struct_init (handle, tail_elts, init);
+  SCM_SET_STRUCT_GC_CHAIN (handle, 0);
+  scm_struct_init (handle, layout, data, tail_elts, init);
+  SCM_SET_CELL_WORD_0 (handle, (scm_t_bits) SCM_STRUCT_DATA (vtable) + scm_tc3_cons_gloc);
   SCM_ALLOW_INTS;
   return handle;
 }
@@ -407,89 +479,79 @@ SCM_DEFINE (scm_make_struct, "make-struct", 2, 0, 1,
 
 
 SCM_DEFINE (scm_make_vtable_vtable, "make-vtable-vtable", 2, 0, 1,
-            (SCM extra_fields, SCM tail_array_size, SCM init),
+            (SCM user_fields, SCM tail_array_size, SCM init),
 	    "Return a new, self-describing vtable structure.\n\n"
-	    "@var{new-fields} is a layout specification describing fields\n"
-	    "of the resulting structure beginning at the position bound to\n"
-	    "@code{vtable-offset-user}.\n\n"
+	    "@var{user-fields} is a string describing user defined fields of the\n"
+	    "vtable beginning at index @code{vtable-offset-user}\n"
+	    "(see @code{make-struct-layout}).\n\n"
 	    "@var{tail-size} specifies the size of the tail-array (if any) of\n"
 	    "this vtable.\n\n"
-	    "@var{inits} initializes the fields of the vtable.  Minimally, one\n"
-	    "initializer must be provided: the layout specification for instances\n"
-	    "of the type this vtable will describe.  If a second initializer is\n"
-	    "provided, it will be interpreted as a print call-back function.\n\n"
-	    "@example\n"
-	    ";;; loading ,a...\n"
-	    "(define x\n"
-	    "  (make-vtable-vtable (make-struct-layout (quote pw))\n"
-	    "                      0\n"
-	    "                      'foo))\n\n"
-	    "(struct? x)\n"
-	    "@result{} #t\n"
-	    "(struct-vtable? x)\n"
-	    "@result{} #t\n"
-	    "(eq? x (struct-vtable x))\n"
-	    "@result{} #t\n"
-	    "(struct-ref x vtable-offset-user)\n"
-	    "@result{} foo\n"
-	    "(struct-ref x 0)\n"
-	    "@result{} pruosrpwpw\n\n\n"
-	    "(define y\n"
-	    "  (make-struct x\n"
-	    "               0\n"
-	    "               (make-struct-layout (quote pwpwpw))\n"
-	    "               'bar))\n\n"
-	    "(struct? y)\n"
-	    "@result{} #t\n"
-	    "(struct-vtable? y)\n"
-	    "@result{} #t\n"
-	    "(eq? x y)\n"
-	    "@result{} ()\n"
-	    "(eq? x (struct-vtable y))\n"
-	    "@result{} #t\n"
-	    "(struct-ref y 0)\n"
-	    "@result{} pwpwpw\n"
-	    "(struct-ref y vtable-offset-user)\n"
-	    "@result{} bar\n\n\n"
-	    "(define z (make-struct y 0 'a 'b 'c))\n\n"
-	    "(struct? z)\n"
-	    "@result{} #t\n"
-	    "(struct-vtable? z)\n"
-	    "@result{} ()\n"
-	    "(eq? y (struct-vtable z))\n"
-	    "@result{} #t\n"
-	    "(map (lambda (n) (struct-ref z n)) '(0 1 2))\n"
-	    "@result{} (a b c)\n"
-	    "@end example\n"
-	    "")
+	    "@var{init1}, @dots{} are the optional initializers for the fields of\n"
+	    "the vtable.\n\n"
+	    "Vtables have one initializable system field---the struct printer.\n"
+	    "This field comes before the user fields in the initializers passed\n"
+	    "to @code{make-vtable-vtable} and @code{make-struct}, and thus works as\n"
+	    "a third optional argument to @code{make-vtable-vtable} and a fourth to\n"
+	    "@code{make-struct} when creating vtables:\n\n"
+	    "If the value is a procedure, it will be called instead of the standard\n"
+	    "printer whenever a struct described by this vtable is printed.\n"
+	    "The procedure will be called with arguments STRUCT and PORT.\n\n"
+	    "The structure of a struct is described by a vtable, so the vtable is\n"
+	    "in essence the type of the struct.  The vtable is itself a struct with\n"
+	    "a vtable.  This could go on forever if it weren't for the\n"
+	    "vtable-vtables which are self-describing vtables, and thus terminate\n"
+	    "the chain.\n\n"
+	    "There are several potential ways of using structs, but the standard\n"
+	    "one is to use three kinds of structs, together building up a type\n"
+	    "sub-system: one vtable-vtable working as the root and one or several\n"
+	    "\"types\", each with a set of \"instances\".  (The vtable-vtable should be\n"
+	    "compared to the class <class> which is the class of itself.)\n\n"
+	    "@lisp\n"
+	    "(define ball-root (make-vtable-vtable \"pr\" 0))\n\n"
+	    "(define (make-ball-type ball-color)\n"
+	    "  (make-struct ball-root 0\n"
+	    "	       (make-struct-layout \"pw\")\n"
+	    "               (lambda (ball port)\n"
+	    "                 (format port \"#<a ~A ball owned by ~A>\"\n"
+	    "                         (color ball)\n"
+	    "                         (owner ball)))\n"
+	    "               ball-color))\n"
+	    "(define (color ball) (struct-ref (struct-vtable ball) vtable-offset-user))\n"
+	    "(define (owner ball) (struct-ref ball 0))\n\n"
+	    "(define red (make-ball-type 'red))\n"
+	    "(define green (make-ball-type 'green))\n\n"
+	    "(define (make-ball type owner) (make-struct type 0 owner))\n\n"
+	    "(define ball (make-ball green 'Nisse))\n"
+	    "ball @result{} #<a green ball owned by Nisse>\n"
+	    "@end lisp")
 #define FUNC_NAME s_scm_make_vtable_vtable
 {
   SCM fields;
   SCM layout;
   int basic_size;
   int tail_elts;
-  scm_bits_t * data;
+  scm_t_bits * data;
   SCM handle;
 
-  SCM_VALIDATE_ROSTRING (1,extra_fields);
-  SCM_VALIDATE_INUM (2,tail_array_size);
+  SCM_VALIDATE_STRING (1, user_fields);
+  SCM_VALIDATE_INUM (2, tail_array_size);
   SCM_VALIDATE_REST_ARGUMENT (init);
 
-  fields = scm_string_append (scm_listify (required_vtable_fields,
-					   extra_fields,
-					   SCM_UNDEFINED));
+  fields = scm_string_append (scm_list_2 (required_vtable_fields,
+					  user_fields));
   layout = scm_make_struct_layout (fields);
-  basic_size = SCM_LENGTH (layout) / 2;
+  basic_size = SCM_SYMBOL_LENGTH (layout) / 2;
   tail_elts = SCM_INUM (tail_array_size);
-  SCM_NEWCELL (handle);
+  SCM_NEWCELL2 (handle);
   SCM_DEFER_INTS;
   data = scm_alloc_struct (basic_size + tail_elts,
 			   scm_struct_n_extra_words,
 			   "make-vtable-vtable");
   SCM_SET_CELL_WORD_1 (handle, data);
-  SCM_SET_CELL_WORD_0 (handle, (scm_bits_t) data + scm_tc3_cons_gloc);
-  SCM_SET_STRUCT_LAYOUT (handle, layout);
-  scm_struct_init (handle, tail_elts, scm_cons (layout, init));
+  SCM_SET_STRUCT_GC_CHAIN (handle, 0);
+  data [scm_vtable_index_layout] = SCM_UNPACK (layout);
+  scm_struct_init (handle, layout, data, tail_elts, scm_cons (layout, init));
+  SCM_SET_CELL_WORD_0 (handle, (scm_t_bits) data + scm_tc3_cons_gloc);
   SCM_ALLOW_INTS;
   return handle;
 }
@@ -500,7 +562,7 @@ SCM_DEFINE (scm_make_vtable_vtable, "make-vtable-vtable", 2, 0, 1,
 
 SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
             (SCM handle, SCM pos),
-	    "@deffnx primitive struct-set! struct n value\n"
+	    "@deffnx {Scheme Procedure} struct-set! struct n value\n"
 	    "Access (or modify) the @var{n}th field of @var{struct}.\n\n"
 	    "If the field is of type 'p', then it can be set to an arbitrary value.\n\n"
 	    "If the field is of type 'u', then it can only be set to a non-negative\n"
@@ -508,12 +570,12 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
 #define FUNC_NAME s_scm_struct_ref
 {
   SCM answer = SCM_UNDEFINED;
-  scm_bits_t * data;
+  scm_t_bits * data;
   SCM layout;
   int p;
-  scm_bits_t n_fields;
-  unsigned char * fields_desc;
-  unsigned char field_type = 0;
+  scm_t_bits n_fields;
+  char * fields_desc;
+  char field_type = 0;
   
 
   SCM_VALIDATE_STRUCT (1,handle);
@@ -523,14 +585,14 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
   data = SCM_STRUCT_DATA (handle);
   p = SCM_INUM (pos);
 
-  fields_desc = (unsigned char *) SCM_CHARS (layout);
+  fields_desc = SCM_SYMBOL_CHARS (layout);
   n_fields = data[scm_struct_i_n_words];
   
   SCM_ASSERT_RANGE(1,pos, p < n_fields);
 
-  if (p * 2 < SCM_LENGTH (layout))
+  if (p * 2 < SCM_SYMBOL_LENGTH (layout))
     {
-      unsigned char ref;
+      char ref;
       field_type = fields_desc[p * 2];
       ref = fields_desc[p * 2 + 1];
       if ((ref != 'r') && (ref != 'w'))
@@ -538,16 +600,13 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
 	  if ((ref == 'R') || (ref == 'W'))
 	    field_type = 'u';
 	  else
-	    SCM_ASSERT (0, pos, "ref denied", FUNC_NAME);
+	    SCM_MISC_ERROR ("ref denied for field ~A", scm_list_1 (pos));
 	}
     }
-  else if (fields_desc[SCM_LENGTH (layout) - 1] != 'O')    
-    field_type = fields_desc[SCM_LENGTH (layout) - 2];
+  else if (fields_desc[SCM_SYMBOL_LENGTH (layout) - 1] != 'O')    
+    field_type = fields_desc[SCM_SYMBOL_LENGTH (layout) - 2];
   else
-    {
-      SCM_ASSERT (0, pos, "ref denied", FUNC_NAME);
-      abort ();
-    }
+    SCM_MISC_ERROR ("ref denied for field ~A", scm_list_1 (pos));
   
   switch (field_type)
     {
@@ -572,8 +631,8 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
 
 
     default:
-      SCM_ASSERT (0, SCM_MAKE_CHAR (field_type), "unrecognized field type", FUNC_NAME);
-      break;
+      SCM_MISC_ERROR ("unrecognized field type: ~S",
+		      scm_list_1 (SCM_MAKE_CHAR (field_type)));
     }
 
   return answer;
@@ -583,15 +642,17 @@ SCM_DEFINE (scm_struct_ref, "struct-ref", 2, 0, 0,
 
 SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
             (SCM handle, SCM pos, SCM val),
-	    "")
+	    "Set the slot of the structure @var{handle} with index @var{pos}\n"
+	    "to @var{val}.  Signal an error if the slot can not be written\n"
+	    "to.")
 #define FUNC_NAME s_scm_struct_set_x
 {
-  scm_bits_t * data;
+  scm_t_bits * data;
   SCM layout;
   int p;
   int n_fields;
-  unsigned char * fields_desc;
-  unsigned char field_type = 0;
+  char * fields_desc;
+  char field_type = 0;
 
   SCM_VALIDATE_STRUCT (1,handle);
   SCM_VALIDATE_INUM (2,pos);
@@ -600,26 +661,23 @@ SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
   data = SCM_STRUCT_DATA (handle);
   p = SCM_INUM (pos);
 
-  fields_desc = (unsigned char *)SCM_CHARS (layout);
+  fields_desc = SCM_SYMBOL_CHARS (layout);
   n_fields = data[scm_struct_i_n_words];
 
   SCM_ASSERT_RANGE (1,pos, p < n_fields);
 
-  if (p * 2 < SCM_LENGTH (layout))
+  if (p * 2 < SCM_SYMBOL_LENGTH (layout))
     {
-      unsigned char set_x;
+      char set_x;
       field_type = fields_desc[p * 2];
       set_x = fields_desc [p * 2 + 1];
       if (set_x != 'w')
-	SCM_ASSERT (0, pos, "set_x denied", FUNC_NAME);
+	SCM_MISC_ERROR ("set! denied for field ~A", scm_list_1 (pos));
     }
-  else if (fields_desc[SCM_LENGTH (layout) - 1] == 'W')    
-    field_type = fields_desc[SCM_LENGTH (layout) - 2];
+  else if (fields_desc[SCM_SYMBOL_LENGTH (layout) - 1] == 'W')    
+    field_type = fields_desc[SCM_SYMBOL_LENGTH (layout) - 2];
   else
-    {
-      SCM_ASSERT (0, pos, "set_x denied", FUNC_NAME);
-      abort ();
-    }
+    SCM_MISC_ERROR ("set! denied for field ~A", scm_list_1 (pos));
   
   switch (field_type)
     {
@@ -629,7 +687,7 @@ SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
 
 #if 0
     case 'i':
-      data[p] = SCM_NUM2LONG (3,val);
+      data[p] = SCM_NUM2LONG (3, val);
       break;
 
     case 'd':
@@ -642,12 +700,11 @@ SCM_DEFINE (scm_struct_set_x, "struct-set!", 3, 0, 0,
       break;
 
     case 's':
-      SCM_ASSERT (0, SCM_MAKE_CHAR (field_type), "self fields immutable", FUNC_NAME);
-      break;
+      SCM_MISC_ERROR ("self fields immutable", SCM_EOL);
 
     default:
-      SCM_ASSERT (0, SCM_MAKE_CHAR (field_type), "unrecognized field type", FUNC_NAME);
-      break;
+      SCM_MISC_ERROR ("unrecognized field type: ~S",
+		      scm_list_1 (SCM_MAKE_CHAR (field_type)));
     }
 
   return val;
@@ -668,7 +725,7 @@ SCM_DEFINE (scm_struct_vtable, "struct-vtable", 1, 0, 0,
 
 SCM_DEFINE (scm_struct_vtable_tag, "struct-vtable-tag", 1, 0, 0, 
             (SCM handle),
-	    "")
+	    "Return the vtable tag of the structure @var{handle}.")
 #define FUNC_NAME s_scm_struct_vtable_tag
 {
   SCM_VALIDATE_VTABLE (1,handle);
@@ -683,8 +740,8 @@ SCM_DEFINE (scm_struct_vtable_tag, "struct-vtable-tag", 1, 0, 0,
  * how to associate names with vtables.
  */
 
-unsigned int
-scm_struct_ihashq (SCM obj, unsigned int n)
+unsigned long
+scm_struct_ihashq (SCM obj, unsigned long n)
 {
   /* The length of the hash table should be a relative prime it's not
      necessary to shift down the address.  */
@@ -707,7 +764,7 @@ scm_struct_create_handle (SCM obj)
 
 SCM_DEFINE (scm_struct_vtable_name, "struct-vtable-name", 1, 0, 0, 
             (SCM vtable),
-	    "")
+	    "Return the name of the vtable @var{vtable}.")
 #define FUNC_NAME s_scm_struct_vtable_name
 {
   SCM_VALIDATE_VTABLE (1,vtable);
@@ -717,7 +774,7 @@ SCM_DEFINE (scm_struct_vtable_name, "struct-vtable-name", 1, 0, 0,
 
 SCM_DEFINE (scm_set_struct_vtable_name_x, "set-struct-vtable-name!", 2, 0, 0, 
             (SCM vtable, SCM name),
-	    "")
+	    "Set the name of the vtable @var{vtable} to @var{name}.")
 #define FUNC_NAME s_scm_set_struct_vtable_name_x
 {
   SCM_VALIDATE_VTABLE (1,vtable);
@@ -753,17 +810,25 @@ scm_print_struct (SCM exp, SCM port, scm_print_state *pstate)
     }
 }
 
+void
+scm_struct_prehistory ()
+{
+  scm_c_hook_add (&scm_before_mark_c_hook, scm_struct_gc_init, 0, 0);
+  scm_c_hook_add (&scm_after_sweep_c_hook, scm_free_structs, 0, 0);
+}
+
 void
 scm_init_struct ()
 {
   scm_struct_table
     = scm_permanent_object (scm_make_weak_key_hash_table (SCM_MAKINUM (31)));
-  required_vtable_fields = SCM_CAR (scm_intern_obarray ("pruosrpw", sizeof ("pruosrpw") - 1, SCM_BOOL_F));
+  required_vtable_fields = scm_makfrom0str ("pruosrpw");
   scm_permanent_object (required_vtable_fields);
-  scm_sysintern ("vtable-index-layout", SCM_MAKINUM (scm_vtable_index_layout));
-  scm_sysintern ("vtable-index-vtable", SCM_MAKINUM (scm_vtable_index_vtable));
-  scm_sysintern ("vtable-index-printer", SCM_MAKINUM (scm_vtable_index_printer));
-  scm_sysintern ("vtable-offset-user", SCM_MAKINUM (scm_vtable_offset_user));
+  scm_c_define ("vtable-index-layout", SCM_MAKINUM (scm_vtable_index_layout));
+  scm_c_define ("vtable-index-vtable", SCM_MAKINUM (scm_vtable_index_vtable));
+  scm_c_define ("vtable-index-printer",
+		SCM_MAKINUM (scm_vtable_index_printer));
+  scm_c_define ("vtable-offset-user", SCM_MAKINUM (scm_vtable_offset_user));
 #include "libguile/struct.x"
 }
 
