@@ -3,7 +3,7 @@
 #ifndef THREADSH
 #define THREADSH
 
-/* Copyright (C) 1996, 1997, 1998, 2000, 2002 Free Software Foundation, Inc.
+/* Copyright (C) 1996, 1997, 1998, 2000, 2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -17,8 +17,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -53,17 +53,17 @@
 
 
 /* smob tags for the thread datatypes */
-extern long scm_tc16_thread;
-extern long scm_tc16_mutex;
-extern long scm_tc16_condvar;
+extern scm_t_bits scm_tc16_thread;
+extern scm_t_bits scm_tc16_mutex;
+extern scm_t_bits scm_tc16_condvar;
 
-#define SCM_THREADP(x) (!SCM_IMP (x) && (SCM_TYP16 (x) == scm_tc16_thread))
-#define SCM_THREAD_DATA(x) ((void *) SCM_CELL_WORD_1 (x))
+#define SCM_THREADP(x)      SCM_TYP16_PREDICATE (scm_tc16_thread, x)
+#define SCM_THREAD_DATA(x)  ((void *) SCM_CELL_WORD_1 (x))
 
-#define SCM_MUTEXP(x) (!SCM_IMP (x) && (SCM_TYP16 (x) == scm_tc16_mutex))
-#define SCM_MUTEX_DATA(x) ((void *) SCM_CELL_WORD_1 (x))
+#define SCM_MUTEXP(x)       SCM_TYP16_PREDICATE (scm_tc16_mutex, x)
+#define SCM_MUTEX_DATA(x)   ((void *) SCM_CELL_WORD_1 (x))
 
-#define SCM_CONDVARP(x) (!SCM_IMP (x) && (SCM_TYP16 (x) == scm_tc16_condvar))
+#define SCM_CONDVARP(x)     SCM_TYP16_PREDICATE (scm_tc16_condvar, x)
 #define SCM_CONDVAR_DATA(x) ((void *) SCM_CELL_WORD_1 (x))
 
 /* Initialize implementation specific details of the threads support */
@@ -77,8 +77,8 @@ SCM scm_threads_lock_mutex (SCM);
 SCM scm_threads_unlock_mutex (SCM);
 SCM scm_threads_monitor (void);
 
-SCM scm_spawn_thread (scm_catch_body_t body, void *body_data,
-		      scm_catch_handler_t handler, void *handler_data);
+SCM scm_spawn_thread (scm_t_catch_body body, void *body_data,
+		      scm_t_catch_handler handler, void *handler_data);
 
 /* These are versions of the ordinary sleep and usleep functions,
    that play nicely with the thread system.  */
