@@ -1,8 +1,8 @@
 /* classes: h_files */
 
-#ifndef VECTORSH
-#define VECTORSH
-/*	Copyright (C) 1995, 1996, 1998, 2000, 2002 Free Software Foundation, Inc.
+#ifndef SCM_VECTORS_H
+#define SCM_VECTORS_H
+/* Copyright (C) 1995,1996,1998,2000,2001 Free Software Foundation, Inc.
  * 
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,8 +16,8 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this software; see the file COPYING.  If not, write to
- * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA 02111-1307 USA
+ * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+ * Boston, MA 02110-1301 USA
  *
  * As a special exception, the Free Software Foundation gives permission
  * for additional uses of the text contained in its release of GUILE.
@@ -50,6 +50,12 @@
 
 
 #define SCM_VECTORP(x) (SCM_NIMP (x) && (SCM_TYP7S (x) == scm_tc7_vector))
+#define SCM_VECTOR_BASE(x) ((scm_t_bits *) SCM_CELL_WORD_1 (x))
+#define SCM_SET_VECTOR_BASE(v, b) (SCM_SET_CELL_WORD_1 ((v), (b)))
+#define SCM_VECTOR_MAX_LENGTH ((1L << 24) - 1)
+#define SCM_VECTOR_LENGTH(x) (((unsigned long) SCM_CELL_WORD_0 (x)) >> 8)
+#define SCM_SET_VECTOR_LENGTH(v, l, t) (SCM_SET_CELL_WORD_0 ((v), ((l) << 8) + (t)))
+
 #define SCM_VELTS(x) ((SCM *) SCM_CELL_WORD_1 (x))
 #define SCM_VELTS_AS_STACKITEMS(x) ((SCM_STACKITEM *) SCM_CELL_WORD_1 (x))
 #define SCM_SETVELTS(x,v) (SCM_SET_CELL_WORD_1 ((x), (v)))
@@ -59,14 +65,15 @@
 /*
   bit vectors
  */
-#define SCM_BITVEC_REF(a, i) ((SCM_UNPACK(SCM_VELTS(a)[(i)/SCM_LONG_BIT]) & (1L<<((i)%SCM_LONG_BIT))) ? 1 : 0)
-#define SCM_BITVEC_SET(a, i) SCM_UNPACK(SCM_VELTS(a)[(i)/SCM_LONG_BIT]) |= (1L<<((i)%SCM_LONG_BIT))
-#define SCM_BITVEC_CLR(a, i) SCM_UNPACK(SCM_VELTS(a)[(i)/SCM_LONG_BIT]) &= ~(1L<<((i)%SCM_LONG_BIT))
+#define SCM_BITVEC_REF(a, i) ((SCM_BITVECTOR_BASE (a) [(i) / SCM_LONG_BIT] & (1L << ((i) % SCM_LONG_BIT))) ? 1 : 0)
+#define SCM_BITVEC_SET(a, i) SCM_BITVECTOR_BASE (a) [(i) / SCM_LONG_BIT] |= (1L << ((i) % SCM_LONG_BIT))
+#define SCM_BITVEC_CLR(a, i) SCM_BITVECTOR_BASE (a) [(i) / SCM_LONG_BIT] &= ~(1L << ((i) % SCM_LONG_BIT))
 
 
 
 
-extern SCM scm_vector_set_length_x (SCM vect, SCM len);
+extern SCM scm_c_make_vector (unsigned long int k, SCM fill);
+
 extern SCM scm_vector_p (SCM x);
 extern SCM scm_vector_length (SCM v);
 extern SCM scm_vector (SCM l);
@@ -86,11 +93,11 @@ extern void scm_init_vectors (void);
 
 #if (SCM_DEBUG_DEPRECATED == 0)
 
-#define SCM_NVECTORP(x) (!SCM_VECTORP (x))
+extern SCM scm_vector_set_length_x (SCM vect, SCM len);
 
 #endif  /* SCM_DEBUG_DEPRECATED == 0 */
 
-#endif  /* VECTORSH */
+#endif  /* SCM_VECTORS_H */
 
 /*
   Local Variables:
