@@ -1,6 +1,6 @@
 ;;; srfi-1.scm --- List Library
 
-;; 	Copyright (C) 2001, 2002, 2004 Free Software Foundation, Inc.
+;; 	Copyright (C) 2001, 2002, 2004, 2005 Free Software Foundation, Inc.
 ;;
 ;; This program is free software; you can redistribute it and/or
 ;; modify it under the terms of the GNU General Public License as
@@ -14,8 +14,8 @@
 ;;
 ;; You should have received a copy of the GNU General Public License
 ;; along with this software; see the file COPYING.  If not, write to
-;; the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
-;; Boston, MA 02111-1307 USA
+;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+;; Boston, MA 02110-1301 USA
 ;;
 ;; As a special exception, the Free Software Foundation gives permission
 ;; for additional uses of the text contained in its release of GUILE.
@@ -360,11 +360,10 @@
 	     (and (elt= (car a) (car b))
 		  (lp (cdr a) (cdr b)))))))
   (or (null? rest)
-      (let ((first (car rest)))
-	(let lp ((lists rest))
-	  (or (null? lists)
-	      (and (lists-equal first (car lists))
-		   (lp (cdr lists))))))))
+      (let lp ((lists rest))
+	(or (null? (cdr lists))
+	    (and (lists-equal (car lists) (cadr lists))
+		 (lp (cdr lists)))))))
 
 ;;; Selectors
 
@@ -620,10 +619,14 @@
 	  (uf (g seed) (cons (f seed) lis))))))
 
 (define (reduce f ridentity lst)
-  (fold f ridentity lst))
+  (if (null? lst)
+      ridentity
+      (fold f (car lst) (cdr lst))))
 
 (define (reduce-right f ridentity lst)
-  (fold-right f ridentity lst))
+  (if (null? lst)
+      ridentity
+      (fold-right f (last lst) (drop-right lst 1))))
 
 
 ;; Internal helper procedure.  Map `f' over the single list `ls'.
@@ -961,33 +964,41 @@
 	  (and (every (lambda (el) (member el (car r) =)) f)
 	       (lp (car r) (cdr r)))))))
 
-(define (lset= = list1 . rest)
+(define (lset= = . rest)
   (if (null? rest)
     #t
-    (let lp ((f list1) (r rest))
+    (let lp ((f (car rest)) (r (cdr rest)))
       (or (null? r)
 	  (and (every (lambda (el) (member el (car r) =)) f)
-	       (every (lambda (el) (member el f =)) (car r))
+	       (every (lambda (el) (member el f (lambda (x y) (= y x)))) (car r))
 	       (lp (car r) (cdr r)))))))
 
+;; It's not quite clear if duplicates among the `rest' elements are meant to
+;; be cast out.  The spec says `=' is called as (= lstelem restelem),
+;; suggesting perhaps not, but the reference code shows the "list" at each
+;; stage as including those elements already added.  The latter corresponds
+;; to what's described for lset-union, so that's what's done.
+;;
 (define (lset-adjoin = list . rest)
   (let lp ((l rest) (acc list))
     (if (null? l)
       acc
-      (if (member (car l) acc)
+      (if (member (car l) acc (lambda (x y) (= y x)))
 	(lp (cdr l) acc)
 	(lp (cdr l) (cons (car l) acc))))))
 
 (define (lset-union = . rest)
-  (let lp0 ((l rest) (acc '()))
-    (if (null? l)
-      (reverse! acc)
-      (let lp1 ((ll (car l)) (acc acc))
-	(if (null? ll)
-	  (lp0 (cdr l) acc)
-	  (if (member (car ll) acc =)
-	    (lp1 (cdr ll) acc)
-	    (lp1 (cdr ll) (cons (car ll) acc))))))))
+  (let ((acc '()))
+    (for-each (lambda (lst)
+		(if (null? acc)
+		    (set! acc lst)
+		    (for-each (lambda (elem)
+				(if (not (member elem acc
+						 (lambda (x y) (= y x))))
+				    (set! acc (cons elem acc))))
+			      lst)))
+	      rest)
+    acc))
 
 (define (lset-intersection = list1 . rest)
   (let lp ((l list1) (acc '()))
